@@ -1,41 +1,33 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from collections import defaultdict
-import time
+import json
 import logging
 import os
-import json
+import time
 import warnings
+from collections import defaultdict
 from datetime import datetime
 from decimal import Decimal
 from itertools import chain, repeat
 from threading import RLock
 
-try:
-    from psycopg2 import connect
-except ImportError:
-    from psycopg2cffi import compat
-    compat.register()
-except ImportError:
-    from psycopg2 import connect
-from psycopg2 import Binary
+from psycopg2 import Binary, connect
+from psycopg2.extensions import (
+    ISOLATION_LEVEL_REPEATABLE_READ, UNICODE, AsIs, cursor, register_adapter,
+    register_type)
+from psycopg2.pool import PoolError, ThreadedConnectionPool
 from psycopg2.sql import SQL, Identifier
-from psycopg2.pool import ThreadedConnectionPool, PoolError
-from psycopg2.extensions import cursor
-from psycopg2.extensions import ISOLATION_LEVEL_REPEATABLE_READ
-from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT
-from psycopg2.extensions import register_type, register_adapter
-from psycopg2.extensions import UNICODE, AsIs
+
 try:
-    from psycopg2.extensions import PYDATE, PYDATETIME, PYTIME, PYINTERVAL
+    from psycopg2.extensions import PYDATE, PYDATETIME, PYINTERVAL, PYTIME
 except ImportError:
     PYDATE, PYDATETIME, PYTIME, PYINTERVAL = None, None, None, None
+from psycopg2 import DataError as DatabaseDataError
 from psycopg2 import IntegrityError as DatabaseIntegrityError
 from psycopg2 import OperationalError as DatabaseOperationalError
 from psycopg2 import ProgrammingError
 from psycopg2.extras import register_default_json, register_default_jsonb
-
-from sql import Flavor, Cast, For, Table
+from sql import Cast, Flavor, For, Table
 from sql.conditionals import Coalesce
 from sql.functions import Function
 from sql.operators import BinaryOperator, Concat
@@ -44,7 +36,9 @@ from trytond.backend.database import DatabaseInterface, SQLType
 from trytond.config import config, parse_uri
 from trytond.tools.gevent import is_gevent_monkey_patched
 
-__all__ = ['Database', 'DatabaseIntegrityError', 'DatabaseOperationalError']
+__all__ = [
+    'Database',
+    'DatabaseIntegrityError', 'DatabaseDataError', 'DatabaseOperationalError']
 
 logger = logging.getLogger(__name__)
 
@@ -82,12 +76,13 @@ class ForSkipLocked(For):
 
 class Unaccent(Function):
     __slots__ = ()
-    _function = 'unaccent'
+    _function = config.get('database', 'unaccent_function', default='unaccent')
 
 
 class Similarity(Function):
     __slots__ = ()
-    _function = 'similarity'
+    _function = config.get(
+        'database', 'similarity_function', default='similarity')
 
 
 class Match(BinaryOperator):
@@ -186,6 +181,7 @@ class JSONContains(BinaryOperator):
 
 class Database(DatabaseInterface):
 
+    index_translators = []
     _lock = RLock()
     _databases = defaultdict(dict)
     _connpool = None
@@ -195,18 +191,22 @@ class Database(DatabaseInterface):
     _current_user = None
     _has_returning = None
     _has_select_for_skip_locked = None
-    _has_proc = defaultdict(dict)
+    _has_proc = defaultdict(lambda: defaultdict(dict))
+    _extensions = defaultdict(dict)
     _search_full_text_languages = defaultdict(dict)
     flavor = Flavor(ilike=True)
 
     TYPES_MAPPING = {
-        'INTEGER': SQLType('INT4', 'INT4'),
+        'SMALLINT': SQLType('INT2', 'INT2'),
         'BIGINT': SQLType('INT8', 'INT8'),
-        'FLOAT': SQLType('FLOAT8', 'FLOAT8'),
         'BLOB': SQLType('BYTEA', 'BYTEA'),
         'DATETIME': SQLType('TIMESTAMP', 'TIMESTAMP(0)'),
-        'TIMESTAMP': SQLType('TIMESTAMP', 'TIMESTAMP(6)'),
+        'REAL': SQLType('FLOAT4', 'FLOAT4'),
+        'FLOAT': SQLType('FLOAT8', 'FLOAT8'),
         'FULLTEXT': SQLType('TSVECTOR', 'TSVECTOR'),
+        'INTEGER': SQLType('INT4', 'INT4'),
+        'JSON': SQLType('JSONB', 'JSONB'),
+        'TIMESTAMP': SQLType('TIMESTAMP', 'TIMESTAMP(6)'),
         }
 
     def __new__(cls, name=_default_name):
@@ -221,14 +221,10 @@ class Database(DatabaseInterface):
             if name in databases:
                 inst = databases[name]
             else:
-                if name == _default_name:
-                    minconn = 0
-                else:
-                    minconn = _minconn
                 inst = DatabaseInterface.__new__(cls, name=name)
                 try:
                     inst._connpool = ThreadedConnectionPool(
-                        minconn, _maxconn, **cls._connection_params(name),
+                        _minconn, _maxconn, **cls._connection_params(name),
                         cursor_factory=LoggingCursor)
                 except Exception:
                     logger.error(
@@ -250,6 +246,8 @@ class Database(DatabaseInterface):
             warnings.warn("The path specified in the URI will be overridden")
         params = {
             'dsn': uri._replace(path='/' + name).geturl(),
+            'fallback_application_name': os.environ.get(
+                'TRYTOND_APPNAME', 'trytond'),
             }
         return params
 
@@ -272,19 +270,10 @@ class Database(DatabaseInterface):
                     'connection to "%s" failed', self.name, exc_info=True)
                 raise
             try:
-                # We do not use set_session because psycopg2 < 2.7 and
-                # psycopg2cffi change the default_transaction_* attributes
-                # which breaks external pooling at the transaction level.
-                if autocommit:
-                    conn.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
-                else:
-                    conn.set_isolation_level(ISOLATION_LEVEL_REPEATABLE_READ)
-                # psycopg2cffi does not have the readonly property
-                if hasattr(conn, 'readonly'):
-                    conn.readonly = readonly
-                elif not autocommit and readonly:
-                    cursor = conn.cursor()
-                    cursor.execute('SET TRANSACTION READ ONLY')
+                conn.set_session(
+                    isolation_level=ISOLATION_LEVEL_REPEATABLE_READ,
+                    readonly=readonly,
+                    autocommit=autocommit)
                 # Detect disconnection
                 conn.cursor().execute('SELECT 1')
             except DatabaseOperationalError:
@@ -314,11 +303,14 @@ class Database(DatabaseInterface):
         connection.commit()
         cls._list_cache.clear()
 
-    def drop(self, connection, database_name):
+    @classmethod
+    def drop(cls, connection, database_name):
         cursor = connection.cursor()
         cursor.execute(SQL("DROP DATABASE {}")
             .format(Identifier(database_name)))
-        self.__class__._list_cache.clear()
+        cls._list_cache.clear()
+        cls._has_proc.pop(database_name, None)
+        cls._search_full_text_languages.pop(database_name, None)
 
     def get_version(self, connection):
         version = connection.server_version
@@ -372,17 +364,14 @@ class Database(DatabaseInterface):
                     if (len(line) > 0) and (not line.isspace()):
                         cursor.execute(line)
 
-            for module in ('ir', 'res'):
-                state = 'not activated'
-                if module in ('ir', 'res'):
-                    state = 'to activate'
+            for module in ['ir', 'res']:
                 info = get_module_info(module)
-                cursor.execute('SELECT NEXTVAL(\'ir_module_id_seq\')')
-                module_id = cursor.fetchone()[0]
                 cursor.execute('INSERT INTO ir_module '
-                    '(id, create_uid, create_date, name, state) '
-                    'VALUES (%s, %s, now(), %s, %s)',
-                    (module_id, 0, module, state))
+                    '(create_uid, create_date, name, state) '
+                    'VALUES (%s, now(), %s, %s) '
+                    'RETURNING id',
+                    (0, module, 'to activate'))
+                module_id = cursor.fetchone()[0]
                 for dependency in info.get('depends', []):
                     cursor.execute('INSERT INTO ir_module_dependency '
                         '(create_uid, create_date, module, name) '
@@ -425,22 +414,30 @@ class Database(DatabaseInterface):
                 pass
         return True
 
-    def nextid(self, connection, table):
+    def nextid(self, connection, table, count=1):
         cursor = connection.cursor()
         cursor.execute(
-            "SELECT nextval(format(%s, %s))", ('%I', table + '_id_seq',))
-        return cursor.fetchone()[0]
+            "SELECT nextval(pg_get_serial_sequence(format(%s, %s), %s)) "
+            "FROM generate_series(1, %s)",
+            ('%I', table, 'id', count))
+        if count == 1:
+            return cursor.fetchone()[0]
+        else:
+            return [id for id, in cursor]
 
     def setnextid(self, connection, table, value):
         cursor = connection.cursor()
         cursor.execute(
-            "SELECT setval(format(%s, %s), %s)",
-            ('%I', table + '_id_seq', value))
+            "SELECT setval(pg_get_serial_sequence(format(%s, %s), %s), %s)",
+            ('%I', table, 'id', value))
 
     def currid(self, connection, table):
         cursor = connection.cursor()
-        cursor.execute(SQL("SELECT last_value FROM {}").format(
-                Identifier(table + '_id_seq')))
+        cursor.execute(
+            "SELECT pg_get_serial_sequence(format(%s, %s), %s)",
+            ('%I', table, 'id'))
+        sequence_name, = cursor.fetchone()
+        cursor.execute(f"SELECT last_value FROM {sequence_name}")
         return cursor.fetchone()[0]
 
     def lock(self, connection, table):
@@ -535,25 +532,31 @@ class Database(DatabaseInterface):
     def has_sequence(cls):
         return True
 
-    def has_proc(self, name):
-        if name in self._has_proc[self.name]:
-            return self._has_proc[self.name][name]
+    def has_proc(self, name, property='oid'):
+        if (name in self._has_proc[self.name]
+                and property in self._has_proc[self.name][name]):
+            return self._has_proc[self.name][name][property]
         connection = self.get_connection()
         result = False
         try:
             cursor = connection.cursor()
             cursor.execute(
-                "SELECT 1 FROM pg_proc WHERE proname=%s",
-                (name,))
-            result = bool(cursor.rowcount)
+                SQL('SELECT {} FROM pg_proc WHERE proname=%s').format(
+                    Identifier(property)), (name,))
+            result = cursor.fetchone()
+            if result:
+                result, = result
         finally:
             self.put_connection(connection)
-        self._has_proc[self.name][name] = result
+        self._has_proc[self.name][name][property] = result
         return result
 
     def has_unaccent(self):
         return self.has_proc(Unaccent._function)
 
+    def has_unaccent_indexable(self):
+        return self.has_proc(Unaccent._function, 'provolatile') == 'i'
+
     def has_similarity(self):
         return self.has_proc(Similarity._function)
 
@@ -735,6 +738,23 @@ class Database(DatabaseInterface):
     def has_channel(self):
         return True
 
+    def has_extension(self, extension_name):
+        if extension_name in self._extensions[self.name]:
+            return self._extensions[self.name][extension_name]
+
+        connection = self.get_connection()
+        result = False
+        try:
+            cursor = connection.cursor()
+            cursor.execute(
+                "SELECT 1 FROM pg_extension WHERE extname=%s",
+                (extension_name,))
+            result = bool(cursor.rowcount)
+        finally:
+            self.put_connection(connection)
+        self._extensions[self.name][extension_name] = result
+        return result
+
     def json_get(self, column, key=None):
         column = Cast(column, 'jsonb')
         if key:
