@@ -20,7 +20,8 @@ CREATE TABLE ir_model_field (
     field_description VARCHAR,
     ttype VARCHAR,
     help TEXT,
-    module VARCHAR
+    module VARCHAR,
+    "access" BOOLEAN
 );
 
 
@@ -44,7 +45,6 @@ CREATE TABLE ir_translation (
     id INTEGER PRIMARY KEY AUTOINCREMENT,
     lang VARCHAR,
     src TEXT,
-    src_md5 VARCHAR(32) NOT NULL,
     name VARCHAR,
     res_id INTEGER,
     value TEXT,
