@@ -1,17 +1,39 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import argparse
-import os
+import csv
 import logging
 import logging.config
 import logging.handlers
+import os
+import os.path
 from contextlib import contextmanager
+from io import StringIO
 
 from trytond import __version__
 
 logger = logging.getLogger(__name__)
 
 
+def database_completer(parsed_args, **kwargs):
+    from trytond.config import config
+    from trytond.transaction import Transaction
+    config.update_etc(parsed_args.configfile)
+    with Transaction().start(
+            None, 0, readonly=True, close=True) as transaction:
+        return transaction.database.list()
+
+
+def module_completer(**kwargs):
+    from trytond.modules import get_modules
+    return get_modules()
+
+
+def language_completer(**kwargs):
+    files = os.listdir(os.path.join(os.path.dirname(__file__), 'ir', 'locale'))
+    return [os.path.splitext(f)[0] for f in files]
+
+
 def get_base_parser():
     parser = argparse.ArgumentParser()
     parser.add_argument('--version', action='version',
@@ -30,9 +52,18 @@ def get_parser():
     parser.add_argument('--dev', dest='dev', action='store_true',
         help='enable development mode')
 
-    parser.add_argument("-d", "--database", dest="database_names", nargs='+',
-        default=[], metavar='DATABASE', help="specify the database name")
-    parser.add_argument("--logconf", dest="logconf", metavar='FILE',
+    logging_config = os.environ.get('TRYTOND_LOGGING_CONFIG')
+    db_names = os.environ.get('TRYTOND_DATABASE_NAMES')
+    if db_names:
+        db_names = list(next(csv.reader(StringIO(db_names))))
+    else:
+        db_names = []
+    parser.add_argument(
+        "-d", "--database", dest="database_names", nargs='+',
+        default=db_names, metavar='DATABASE',
+        help="specify the database name").completer = database_completer
+    parser.add_argument(
+        "--logconf", dest="logconf", default=logging_config, metavar='FILE',
         help="logging configuration file (ConfigParser format)")
 
     return parser
@@ -42,7 +73,9 @@ def get_parser_daemon():
     parser = get_parser()
     parser.add_argument("--pidfile", dest="pidfile", metavar='FILE',
         help="file where the server pid will be stored")
-    parser.add_argument("--coroutine", action="store_true", dest="coroutine",
+    parser.add_argument(
+        "--coroutine", action="store_true", dest="coroutine",
+        default=bool(os.environ.get('TRYTOND_COROUTINE', False)),
         help="use coroutine for concurrency")
     return parser
 
@@ -70,8 +103,10 @@ def get_parser_cron():
 def get_parser_admin():
     parser = get_parser()
 
-    parser.add_argument("-u", "--update", dest="update", nargs='+', default=[],
-        metavar='MODULE', help="activate or update a module")
+    parser.add_argument(
+        "-u", "--update", dest="update", nargs='+', default=[],
+        metavar='MODULE',
+        help="activate or update a module").completer = module_completer
     parser.add_argument("--all", dest="update", action="append_const",
         const="ir", help="update all activated modules")
     parser.add_argument("--activate-dependencies", dest="activatedeps",
@@ -86,8 +121,10 @@ def get_parser_admin():
         help="Send a test email to the specified address.")
     parser.add_argument("-m", "--update-modules-list", action="store_true",
         dest="update_modules_list", help="Update list of tryton modules")
-    parser.add_argument("-l", "--language", dest="languages", nargs='+',
-        default=[], metavar='CODE', help="Load language translations")
+    parser.add_argument(
+        "-l", "--language", dest="languages", nargs='+',
+        default=[], metavar='CODE',
+        help="Load language translations").completer = language_completer
     parser.add_argument("--hostname", dest="hostname", default=None,
         help="Limit database listing to the hostname")
     parser.add_argument("--validate", dest="validate", nargs='*',
@@ -109,10 +146,17 @@ def get_parser_admin():
 
 def get_parser_console():
     parser = get_base_parser()
-    parser.add_argument("-d", "--database", dest="database_name",
-        required=True, metavar='DATABASE', help="specify the database name")
+    parser.add_argument(
+        "-d", "--database", dest="database_name",
+        required=True, metavar='DATABASE',
+        help="specify the database name").completer = database_completer
     parser.add_argument("--histsize", dest="histsize", type=int, default=500,
         help="The number of commands to remember in the command history")
+    parser.add_argument("--readonly", dest="readonly", action='store_true',
+        help="Start a readonly transaction")
+    parser.add_argument(
+        "--lock-table", dest="lock_tables", nargs='+', default=[],
+        metavar='TABLE', help="Lock tables")
     parser.epilog = "To store changes, `transaction.commit()` must be called."
     return parser
 
@@ -125,13 +169,18 @@ def get_parser_stat():
 
 def config_log(options):
     if options.logconf:
-        logging.config.fileConfig(options.logconf)
+        logging.config.fileConfig(
+            options.logconf, disable_existing_loggers=False)
         logging.getLogger('server').info('using %s as logging '
             'configuration file', options.logconf)
     else:
         logformat = ('%(process)s %(thread)s [%(asctime)s] '
             '%(levelname)s %(name)s %(message)s')
-        level = max(logging.ERROR - options.verbose * 10, logging.NOTSET)
+        if not options.verbose and 'TRYTOND_LOGGING_LEVEL' in os.environ:
+            logging_level = int(os.environ['TRYTOND_LOGGING_LEVEL'])
+            level = max(logging_level, logging.NOTSET)
+        else:
+            level = max(logging.ERROR - options.verbose * 10, logging.NOTSET)
         logging.basicConfig(level=level, format=logformat)
     logging.captureWarnings(True)
 
