@@ -1,18 +1,21 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import datetime
-import time
-from dateutil.relativedelta import relativedelta
 import logging
+import time
+
+from dateutil.relativedelta import relativedelta
 
 from trytond import backend
 from trytond.config import config
+from trytond.exceptions import UserError, UserWarning
 from trytond.model import (
-    ModelView, ModelSQL, DeactivableMixin, fields, dualmethod)
+    DeactivableMixin, Index, ModelSQL, ModelView, dualmethod, fields)
 from trytond.pool import Pool
 from trytond.pyson import Eval
 from trytond.status import processing
-from trytond.transaction import Transaction
+from trytond.tools import timezone as tz
+from trytond.transaction import Transaction, TransactionError
 from trytond.worker import run_task
 
 logger = logging.getLogger(__name__)
@@ -64,21 +67,26 @@ class Cron(DeactivableMixin, ModelSQL, ModelView):
                 ['minutes', 'hours', 'days', 'weeks']),
             },
         depends=['interval_type'])
+    timezone = fields.Function(fields.Char("Timezone"), 'get_timezone')
 
-    next_call = fields.DateTime("Next Call", select=True)
+    next_call = fields.DateTime("Next Call")
     method = fields.Selection([
             ('ir.trigger|trigger_time', "Run On Time Triggers"),
             ('ir.queue|clean', "Clean Task Queue"),
+            ('ir.error|clean', "Clean Errors"),
             ], "Method", required=True)
 
     @classmethod
     def __setup__(cls):
         super(Cron, cls).__setup__()
+        table = cls.__table__()
+
         cls._buttons.update({
                 'run_once': {
                     'icon': 'tryton-launch',
                     },
                 })
+        cls._sql_indexes.add(Index(table, (table.next_call, Index.Range())))
 
     @classmethod
     def __register__(cls, module_name):
@@ -94,9 +102,16 @@ class Cron(DeactivableMixin, ModelSQL, ModelView):
         # Migration from 5.0: remove required on next_call
         table_h.not_null_action('next_call', 'remove')
 
-    @staticmethod
-    def check_xml_record(crons, values):
-        return True
+    @classmethod
+    def default_timezone(cls):
+        return tz.SERVER.tzname(datetime.datetime.now())
+
+    def get_timezone(self, name):
+        return self.default_timezone()
+
+    @classmethod
+    def check_xml_record(cls, crons, values):
+        pass
 
     @classmethod
     def view_attributes(cls):
@@ -107,7 +122,7 @@ class Cron(DeactivableMixin, ModelSQL, ModelView):
             ]
 
     def compute_next_call(self, now):
-        return (now
+        return (now.replace(tzinfo=tz.UTC).astimezone(tz.SERVER)
             + relativedelta(**{self.interval_type: self.interval_number})
             + relativedelta(
                 microsecond=0,
@@ -129,7 +144,7 @@ class Cron(DeactivableMixin, ModelSQL, ModelView):
                     int(self.weekday.index)
                     if self.weekday
                     and self.interval_type not in {'minutes', 'hours', 'days'}
-                    else None)))
+                    else None))).astimezone(tz.UTC).replace(tzinfo=None)
 
     @dualmethod
     @ModelView.button
@@ -146,33 +161,56 @@ class Cron(DeactivableMixin, ModelSQL, ModelView):
         logger.info('cron started for "%s"', db_name)
         now = datetime.datetime.now()
         retry = config.getint('database', 'retry')
-        with transaction.start(db_name, 0, context={'_skip_warnings': True}):
-            transaction.database.lock(transaction.connection, cls._table)
+        with transaction.start(
+                db_name, 0, context={'_skip_warnings': True},
+                _lock_tables=[cls._table]):
+            pool = Pool()
+            Error = pool.get('ir.error')
             crons = cls.search(['OR',
                     ('next_call', '<=', now),
                     ('next_call', '=', None),
                     ])
 
             for cron in crons:
+                def duration():
+                    return (time.monotonic() - started) * 1000
+                started = time.monotonic()
                 name = '<Cron %s@%s %s>' % (cron.id, db_name, cron.method)
-                logger.info("%s started", name)
-                for count in range(retry, -1, -1):
-                    if count != retry:
+                transaction_extras = {}
+                count = 0
+                while True:
+                    if count:
                         time.sleep(0.02 * (retry - count))
                     try:
-                        with processing(name):
-                            cron.run_once()
-                            transaction.commit()
-                    except Exception as e:
-                        transaction.rollback()
-                        if (isinstance(e, backend.DatabaseOperationalError)
-                                and count):
-                            continue
-                        logger.error('%s failed', name, exc_info=True)
+                        with processing(name), \
+                                transaction.new_transaction(
+                                    **transaction_extras) as cron_trans:
+                            try:
+                                cron.run_once()
+                                cron_trans.commit()
+                            except TransactionError as e:
+                                cron_trans.rollback()
+                                e.fix(transaction_extras)
+                                continue
+                            except backend.DatabaseOperationalError:
+                                if count < retry:
+                                    cron_trans.rollback()
+                                    count += 1
+                                    logger.debug("Retry: %i", count)
+                                    continue
+                                else:
+                                    raise
+                    except (UserError, UserWarning) as e:
+                        Error.report(cron, e)
+                        logger.info(
+                            "%s failed after %i ms", name, duration())
+                    except Exception:
+                        logger.exception(
+                            "%s failed after %i ms", name, duration())
                     cron.next_call = cron.compute_next_call(now)
                     cron.save()
-                    transaction.commit()
                     break
+                logger.info("%s in %i ms", name, duration())
         while transaction.tasks:
             task_id = transaction.tasks.pop()
             run_task(db_name, task_id)
