@@ -197,7 +197,7 @@ class Translation(ModelSQL, ModelView):
                     continue
                 if isinstance(val, LazyString):
                     continue
-                assert type not in {'field', 'help'} or not inserted, (
+                assert type != 'field' or not inserted, (
                     "More than one resource "
                     f"for {type} of {name} in {module_name}")
                 cursor.execute(
@@ -211,11 +211,13 @@ class Translation(ModelSQL, ModelView):
             name = model.__name__ + ',' + field_name
             insert(field, 'field', name, field.string)
             insert(field, 'help', name, field.help)
-            if (hasattr(field, 'selection')
-                    and isinstance(field.selection, (tuple, list))
-                    and getattr(field, 'translate_selection', True)):
-                selection = [s for _, s in field.selection]
-                insert(field, 'selection', name, selection)
+            if hasattr(field, 'selection'):
+                if (isinstance(field.selection, (tuple, list))
+                        and getattr(field, 'translate_selection', True)):
+                    selection = [s for _, s in field.selection]
+                    insert(field, 'selection', name, selection)
+                if field.help_selection:
+                    insert(field, 'help', name, field.help_selection.values())
 
     @classmethod
     def register_wizard(cls, wizard, module_name):
@@ -359,7 +361,7 @@ class Translation(ModelSQL, ModelView):
                             try:
                                 field = getattr(
                                     pool.get(record.model.model), record.name)
-                            except KeyError:
+                            except (KeyError, AttributeError):
                                 continue
                             translations[record.id] = ''
                             if ttype == 'field':
@@ -1371,9 +1373,14 @@ class TranslationClean(Wizard):
         field = Model._fields.get(field_name)
         if not field:
             return True
-        if not field.help:
+        help_ = field.help
+        if help_selection := getattr(field, 'help_selection', {}):
+            help_selection = list(help_selection.values())
+        else:
+            help_selection = []
+        if not help_ and not help_selection:
             return True
-        if translation.src not in list(field.help):
+        if translation.src not in list(help_) + help_selection:
             return True
 
     def transition_clean(self):
