@@ -2,7 +2,8 @@
 # this repository contains the full copyright notices and license terms.
 import datetime
 import time
-from sql import Literal, Null, Select
+
+from sql import Literal, Select
 from sql.aggregate import Count, Max
 from sql.functions import CurrentTimestamp
 from sql.operators import Concat
@@ -10,12 +11,12 @@ from sql.operators import Concat
 from trytond.cache import Cache
 from trytond.i18n import gettext
 from trytond.model import (
-    ModelView, ModelSQL, DeactivableMixin, fields, EvalEnvironment, Check)
+    Check, DeactivableMixin, EvalEnvironment, Index, ModelSQL, ModelView,
+    fields)
 from trytond.model.exceptions import ValidationError
 from trytond.pool import Pool
-from trytond.pyson import Eval, PYSONDecoder
-from trytond.tools import grouped_slice
-from trytond.tools import reduce_ids
+from trytond.pyson import Eval, PYSONDecoder, TimeDelta
+from trytond.tools import grouped_slice, reduce_ids
 from trytond.transaction import Transaction
 
 
@@ -27,19 +28,19 @@ class Trigger(DeactivableMixin, ModelSQL, ModelView):
     "Trigger"
     __name__ = 'ir.trigger'
     name = fields.Char('Name', required=True, translate=True)
-    model = fields.Many2One('ir.model', 'Model', required=True, select=True)
-    on_time = fields.Boolean('On Time', select=True, states={
+    model = fields.Many2One('ir.model', 'Model', required=True)
+    on_time = fields.Boolean('On Time', states={
             'invisible': (Eval('on_create', False)
                 | Eval('on_write', False)
                 | Eval('on_delete', False)),
             }, depends=['on_create', 'on_write', 'on_delete'])
-    on_create = fields.Boolean('On Create', select=True, states={
+    on_create = fields.Boolean('On Create', states={
         'invisible': Eval('on_time', False),
         }, depends=['on_time'])
-    on_write = fields.Boolean('On Write', select=True, states={
+    on_write = fields.Boolean('On Write', states={
         'invisible': Eval('on_time', False),
         }, depends=['on_time'])
-    on_delete = fields.Boolean('On Delete', select=True, states={
+    on_delete = fields.Boolean('On Delete', states={
         'invisible': Eval('on_time', False),
         }, depends=['on_time'])
     condition = fields.Char('Condition', required=True,
@@ -48,7 +49,12 @@ class Trigger(DeactivableMixin, ModelSQL, ModelView):
     limit_number = fields.Integer('Limit Number', required=True,
         help='Limit the number of call to "Action Function" by records.\n'
         '0 for no limit.')
-    minimum_time_delay = fields.TimeDelta('Minimum Delay',
+    minimum_time_delay = fields.TimeDelta(
+        "Minimum Delay",
+        domain=['OR',
+            ('minimum_time_delay', '=', None),
+            ('minimum_time_delay', '>=', TimeDelta()),
+            ],
         help='Set a minimum time delay between call to "Action Function" '
         'for the same record.\n'
         'empty for no delay.')
@@ -65,33 +71,17 @@ class Trigger(DeactivableMixin, ModelSQL, ModelView):
                         & ((t.on_create == Literal(True))
                             | (t.on_write == Literal(True))
                             | (t.on_delete == Literal(True))))),
-                '"On Time" and others are mutually exclusive!'),
+                'ir.msg_trigger_exclusive'),
             ]
         cls._order.insert(0, ('name', 'ASC'))
 
     @classmethod
     def __register__(cls, module_name):
-        cursor = Transaction().connection.cursor()
-        table = cls.__table_handler__(module_name)
-        sql_table = cls.__table__()
-
         super(Trigger, cls).__register__(module_name)
 
+        cursor = Transaction().connection.cursor()
         table_h = cls.__table_handler__(module_name)
-
-        # Migration from 3.4:
-        # change minimum_delay into timedelta minimum_time_delay
-        if table.column_exist('minimum_delay'):
-            cursor.execute(*sql_table.select(
-                    sql_table.id, sql_table.minimum_delay,
-                    where=sql_table.minimum_delay != Null))
-            for id_, delay in cursor:
-                delay = datetime.timedelta(hours=delay)
-                cursor.execute(*sql_table.update(
-                        [sql_table.minimum_time_delay],
-                        [delay],
-                        where=sql_table.id == id_))
-            table.drop_column('minimum_delay')
+        sql_table = cls.__table__()
 
         # Migration from 5.4: merge action
         if (table_h.column_exist('action_model')
@@ -109,15 +99,17 @@ class Trigger(DeactivableMixin, ModelSQL, ModelView):
             table_h.drop_column('action_function')
 
     @classmethod
-    def validate(cls, triggers):
-        super(Trigger, cls).validate(triggers)
-        cls.check_condition(triggers)
+    def validate_fields(cls, triggers, field_names):
+        super().validate_fields(triggers, field_names)
+        cls.check_condition(triggers, field_names)
 
     @classmethod
-    def check_condition(cls, triggers):
+    def check_condition(cls, triggers, field_names=None):
         '''
         Check condition
         '''
+        if field_names and 'condition' not in field_names:
+            return
         for trigger in triggers:
             try:
                 PYSONDecoder(noeval=True).decode(trigger.condition)
@@ -227,19 +219,6 @@ class Trigger(DeactivableMixin, ModelSQL, ModelView):
                         new_ids.append(record_id)
             ids = new_ids
 
-        def cast_datetime(value):
-            datepart, timepart = value.split(" ")
-            year, month, day = map(int, datepart.split("-"))
-            timepart_full = timepart.split(".")
-            hours, minutes, seconds = map(
-                int, timepart_full[0].split(":"))
-            if len(timepart_full) == 2:
-                microseconds = int(timepart_full[1])
-            else:
-                microseconds = 0
-            return datetime.datetime(
-                year, month, day, hours, minutes, seconds, microseconds)
-
         # Filter on minimum_time_delay
         if self.minimum_time_delay:
             new_ids = []
@@ -248,7 +227,7 @@ class Trigger(DeactivableMixin, ModelSQL, ModelView):
             cursor.execute(*Select([timestamp_cast(CurrentTimestamp())]))
             now, = cursor.fetchone()
             if isinstance(now, str):
-                now = cast_datetime(now)
+                now = datetime.datetime.fromisoformat(now)
             for sub_ids in grouped_slice(ids):
                 sub_ids = list(sub_ids)
                 red_sql = reduce_ids(trigger_log.record_id, sub_ids)
@@ -263,7 +242,8 @@ class Trigger(DeactivableMixin, ModelSQL, ModelView):
                         continue
                     # SQLite return string for MAX
                     if isinstance(delay[record_id], str):
-                        delay[record_id] = cast_datetime(delay[record_id])
+                        delay[record_id] = datetime.datetime.fromisoformat(
+                            delay[record_id])
                     if now - delay[record_id] >= self.minimum_time_delay:
                         new_ids.append(record_id)
             ids = new_ids
@@ -324,8 +304,13 @@ class TriggerLog(ModelSQL):
     record_id = fields.Integer('Record ID', required=True)
 
     @classmethod
-    def __register__(cls, module_name):
-        super(TriggerLog, cls).__register__(module_name)
+    def __setup__(cls):
+        super().__setup__()
+        cls.__access__.add('trigger')
 
-        table = cls.__table_handler__(module_name)
-        table.index_action(['trigger', 'record_id'], 'add')
+        table = cls.__table__()
+        cls._sql_indexes.add(
+            Index(
+                table,
+                (table.trigger, Index.Equality()),
+                (table.record_id, Index.Range())))
