@@ -1,25 +1,26 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from .model import Model
-from .modelview import ModelView
-from .modelstorage import ModelStorage, EvalEnvironment
-from .modelsingleton import ModelSingleton
-from .modelsql import ModelSQL, Check, Unique, Exclude
-from .workflow import Workflow
-from .dictschema import DictSchemaMixin
-from .match import MatchMixin
-from .union import UnionMixin
-from .symbol import SymbolMixin
-from .multivalue import MultiValueMixin, ValueMixin
-from .descriptors import dualmethod
-from .order import sequence_ordered
 from .active import DeactivableMixin
-from .tree import tree
 from .avatar import avatar_mixin
+from .descriptors import dualmethod
+from .dictschema import DictSchemaMixin
+from .digits import DigitsMixin
+from .match import MatchMixin
+from .model import Model
+from .modelsingleton import ModelSingleton
+from .modelsql import Check, Exclude, Index, ModelSQL, Unique, convert_from
+from .modelstorage import EvalEnvironment, ModelStorage
+from .modelview import ModelView
+from .multivalue import MultiValueMixin, ValueMixin
+from .order import sequence_ordered, sort
+from .symbol import SymbolMixin
+from .tree import sum_tree, tree
+from .union import UnionMixin
+from .workflow import Workflow
 
 __all__ = ['Model', 'ModelView', 'ModelStorage', 'ModelSingleton', 'ModelSQL',
-    'Check', 'Unique', 'Exclude',
+    'Check', 'Unique', 'Exclude', 'Index', 'convert_from',
     'Workflow', 'DictSchemaMixin', 'MatchMixin', 'UnionMixin', 'dualmethod',
-    'MultiValueMixin', 'ValueMixin', 'SymbolMixin',
-    'EvalEnvironment', 'sequence_ordered', 'DeactivableMixin', 'tree',
-    'avatar_mixin']
+    'MultiValueMixin', 'ValueMixin', 'SymbolMixin', 'DigitsMixin',
+    'EvalEnvironment', 'sequence_ordered', 'sort', 'DeactivableMixin', 'tree',
+    'sum_tree', 'avatar_mixin']
