@@ -3,36 +3,47 @@
 from sql import Column, Null
 
 from trytond.filestore import filestore
-from trytond.tools import grouped_slice, reduce_ids
+from trytond.tools import cached_property, grouped_slice, reduce_ids
 from trytond.transaction import Transaction
+
 from .field import Field
 
 
+def caster(d):
+    if isinstance(d, bytes):
+        return d
+    elif isinstance(d, memoryview):
+        return bytes(d)
+    return bytes(d, encoding='utf8')
+
+
 class Binary(Field):
     '''
     Define a binary field (``bytes``).
     '''
     _type = 'binary'
     _sql_type = 'BLOB'
-    cast = bytearray if bytes == str else bytes
+    cast = staticmethod(caster)
 
     def __init__(self, string='', help='', required=False, readonly=False,
-            domain=None, states=None, select=False, on_change=None,
+            domain=None, states=None, on_change=None,
             on_change_with=None, depends=None, context=None, loading='lazy',
             filename=None, file_id=None, store_prefix=None):
         self.filename = filename
-        if filename is not None:
-            if depends is None:
-                depends = [filename]
-            else:
-                depends.append(filename)
         self.file_id = file_id
         self.store_prefix = store_prefix
         super(Binary, self).__init__(string=string, help=help,
             required=required, readonly=readonly, domain=domain, states=states,
-            select=select, on_change=on_change, on_change_with=on_change_with,
+            on_change=on_change, on_change_with=on_change_with,
             depends=depends, context=context, loading=loading)
 
+    @cached_property
+    def display_depends(self):
+        depends = super().display_depends
+        if self.filename:
+            depends.add(self.filename)
+        return depends
+
     def get(self, ids, model, name, values=None):
         '''
         Convert the binary value into ``bytes``
@@ -86,8 +97,6 @@ class Binary(Field):
                 continue
             value = i[name]
             if value:
-                if isinstance(value, str):
-                    value = value.encode('utf-8')
                 value = converter(value)
             else:
                 value = default
