@@ -1,14 +1,18 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+import string
 import warnings
 
+from sql import Expression, Query
 from sql.conditionals import Coalesce, NullIf
+from sql.functions import Trim
 from sql.operators import Not
 
 from trytond.rpc import RPC
-from trytond.tools import unescape_wildcard, is_full_text
+from trytond.tools import is_full_text, unescape_wildcard
 from trytond.transaction import Transaction
-from .field import Field, FieldTranslate, size_validate
+
+from .field import Field, FieldTranslate, order_method, size_validate
 
 
 class Char(FieldTranslate):
@@ -23,8 +27,8 @@ class Char(FieldTranslate):
 
     def __init__(self, string='', size=None, help='', required=False,
             readonly=False, domain=None, states=None, translate=False,
-            select=False, on_change=None, on_change_with=None, depends=None,
-            context=None, loading=None, autocomplete=None):
+            on_change=None, on_change_with=None, depends=None, context=None,
+            loading=None, autocomplete=None, strip=True):
         '''
         :param translate: A boolean. If ``True`` the field is translatable.
         :param size: A integer. If set defines the maximum size of the values.
@@ -32,14 +36,15 @@ class Char(FieldTranslate):
         if loading is None:
             loading = 'lazy' if translate else 'eager'
         super(Char, self).__init__(string=string, help=help, required=required,
-            readonly=readonly, domain=domain, states=states, select=select,
+            readonly=readonly, domain=domain, states=states,
             on_change=on_change, on_change_with=on_change_with,
             depends=depends, context=context, loading=loading)
         self.autocomplete = set()
         if autocomplete:
             warnings.warn('autocomplete argument is deprecated, use the '
                 'depends decorator', DeprecationWarning, stacklevel=2)
-            self.autocomplete |= set(autocomplete)
+            self.autocomplete.update(autocomplete)
+        self.strip = strip
         self.translate = translate
         self.__size = None
         self.size = size
@@ -54,9 +59,51 @@ class Char(FieldTranslate):
 
     size = property(_get_size, _set_size)
 
+    @property
+    def strip(self):
+        return self.__strip
+
+    @strip.setter
+    def strip(self, value):
+        assert value in {False, True, 'leading', 'trailing'}
+        self.__strip = value
+
     @property
     def _sql_type(self):
-        return 'VARCHAR(%s)' % self.size if self.size else 'VARCHAR'
+        if isinstance(self.size, int):
+            return 'VARCHAR(%s)' % self.size
+        else:
+            return 'VARCHAR'
+
+    def __set__(self, inst, value):
+        if isinstance(value, str) and self.strip:
+            if self.strip == 'leading':
+                value = value.lstrip()
+            elif self.strip == 'trailing':
+                value = value.rstrip()
+            else:
+                value = value.strip()
+        super().__set__(inst, value)
+
+    def sql_format(self, value):
+        if value is not None and self.strip:
+            if isinstance(value, (Query, Expression)):
+                if self.strip == 'leading':
+                    position = 'LEADING'
+                elif self.strip == 'trailing':
+                    position = 'TRAILING'
+                else:
+                    position = 'BOTH'
+                value = Trim(
+                    value, position=position, characters=string.whitespace)
+            else:
+                if self.strip == 'leading':
+                    value = value.lstrip()
+                elif self.strip == 'trailing':
+                    value = value.rstrip()
+                else:
+                    value = value.strip()
+        return super().sql_format(value)
 
     def set_rpc(self, model):
         super(Char, self).set_rpc(model)
@@ -140,10 +187,8 @@ class Char(FieldTranslate):
                         expression = fts
         return expression
 
+    @order_method
     def convert_order(self, name, tables, Model):
-        method = getattr(Model, 'order_%s' % name, None)
-        if method:
-            return method(tables)
         transaction = Transaction()
         context = transaction.context
         database = transaction.database
@@ -184,6 +229,7 @@ class Char(FieldTranslate):
     def definition(self, model, language):
         definition = super().definition(model, language)
         definition['autocomplete'] = list(self.autocomplete)
+        definition['strip'] = self.strip
         if self.size is not None:
             definition['size'] = self.size
         return definition
