@@ -1,16 +1,18 @@
 # This file is part of Tryton.  The COPYRIGHT file at the toplevel of this
 # repository contains the full copyright notices and license terms.
-from functools import partial
 import json
+from functools import partial
 
-from sql import operators, Literal, Select, CombiningQuery, Cast, Null
+from sql import Cast, CombiningQuery, Literal, Null, Select, operators
 
 from trytond import backend
 from trytond.pool import Pool
 from trytond.protocols.jsonrpc import JSONDecoder, JSONEncoder
 from trytond.tools import grouped_slice
+from trytond.tools.immutabledict import ImmutableDict
 from trytond.transaction import Transaction
-from .field import Field, SQL_OPERATORS
+
+from .field import SQL_OPERATORS, Field, domain_method, order_method
 
 # Use canonical form
 dumps = partial(
@@ -18,38 +20,18 @@ dumps = partial(
     ensure_ascii=False)
 
 
-class ImmutableDict(dict):
-
-    __slots__ = ()
-
-    def _not_allowed(cls, *args, **kwargs):
-        raise TypeError("Operation not allowed on ImmutableDict")
-
-    __setitem__ = _not_allowed
-    __delitem__ = _not_allowed
-    __ior__ = _not_allowed
-    clear = _not_allowed
-    pop = _not_allowed
-    popitem = _not_allowed
-    setdefault = _not_allowed
-    update = _not_allowed
-
-    del _not_allowed
-
-
 class Dict(Field):
     'Define dict field.'
     _type = 'dict'
-    _sql_type = 'TEXT'
+    _sql_type = 'JSON'
     _py_type = dict
 
     def __init__(self, schema_model, string='', help='', required=False,
-            readonly=False, domain=None, states=None, select=False,
+            readonly=False, domain=None, states=None,
             on_change=None, on_change_with=None, depends=None,
             context=None, loading='lazy'):
         super(Dict, self).__init__(string, help, required, readonly, domain,
-            states, select, on_change, on_change_with, depends, context,
-            loading)
+            states, on_change, on_change_with, depends, context, loading)
         self.schema_model = schema_model
         self.search_unaccented = True
 
@@ -61,6 +43,9 @@ class Dict(Field):
                 # If stored as JSON conversion is done on backend
                 if isinstance(data, str):
                     data = json.loads(data, object_hook=JSONDecoder())
+                for key, val in data.items():
+                    if isinstance(val, list):
+                        data[key] = tuple(val)
                 dicts[value['id']] = ImmutableDict(data)
         return dicts
 
@@ -129,6 +114,7 @@ class Dict(Field):
                 expression &= (column != Null)
         return expression
 
+    @domain_method
     def convert_domain(self, domain, tables, Model):
         name, operator, value = domain[:3]
         if '.' not in name:
@@ -180,6 +166,7 @@ class Dict(Field):
         expression = self._domain_add_null(column, operator, value, expression)
         return expression
 
+    @order_method
     def convert_order(self, name, tables, Model):
         fname, _, key = name.partition('.')
         if not key:
