@@ -1,18 +1,29 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from trytond.pyson import PYSONEncoder, PYSON
-from .field import Field
+from trytond.pyson import PYSON, PYSONEncoder
+from trytond.tools import cached_property
+
+from .field import Field, get_eval_fields
 
 
 def digits_validate(value):
     if value:
-        assert isinstance(value, tuple), 'digits must be a tuple'
-        for i in value:
-            assert isinstance(i, (int, PYSON)), \
-                'digits must be tuple of integers or PYSON'
-            if isinstance(i, PYSON):
-                assert i.types().issubset(set([int, int])), \
-                    'PYSON digits must return an integer'
+        assert isinstance(value, (tuple, str)), \
+                'digits must be a tuple or a string'
+        if isinstance(value, tuple):
+            for i in value:
+                assert isinstance(i, (int, PYSON)), \
+                    'digits must be tuple of integers or PYSON'
+                if isinstance(i, PYSON):
+                    assert i.types() == {int}, \
+                        'PYSON digits must return an integer'
+
+
+def _get_digits_depends(field):
+    if isinstance(field.digits, str):
+        return {field.digits}
+    else:
+        return get_eval_fields(field.digits)
 
 
 class Float(Field):
@@ -24,7 +35,7 @@ class Float(Field):
     _py_type = float
 
     def __init__(self, string='', digits=None, help='', required=False,
-            readonly=False, domain=None, states=None, select=False,
+            readonly=False, domain=None, states=None,
             on_change=None, on_change_with=None, depends=None,
             context=None, loading='eager'):
         '''
@@ -33,7 +44,7 @@ class Float(Field):
         '''
         super(Float, self).__init__(string=string, help=help,
             required=required, readonly=readonly, domain=domain, states=states,
-            select=select, on_change=on_change, on_change_with=on_change_with,
+            on_change=on_change, on_change_with=on_change_with,
             depends=depends, context=context, loading=loading)
         self.__digits = None
         self.digits = digits
@@ -54,3 +65,15 @@ class Float(Field):
         definition = super().definition(model, language)
         definition['digits'] = encoder.encode(self.digits)
         return definition
+
+    @cached_property
+    def display_depends(self):
+        return super().display_depends | _get_digits_depends(self)
+
+    @cached_property
+    def edition_depends(self):
+        return super().edition_depends | _get_digits_depends(self)
+
+    @cached_property
+    def validation_depends(self):
+        return super().validation_depends | _get_digits_depends(self)
