@@ -1,12 +1,12 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from sql import Query, Expression, Null, Literal
-from sql.operators import ILike, Not, In, NotIn
+from sql import Expression, Literal, Null, Query
+from sql.operators import ILike, In, Not, NotIn
 
 from trytond.transaction import Transaction
 
-from .field import Field, SQL_OPERATORS
 from .char import Char
+from .field import SQL_OPERATORS, Field, order_method
 
 
 class Text(Char):
@@ -18,6 +18,9 @@ class Text(Char):
     forbidden_chars = ''
     search_full_text = True
 
+    def __init__(self, *args, strip=False, **kwarg):
+        super().__init__(*args, strip=strip, **kwarg)
+
 
 class FullText(Field):
     '''
@@ -82,10 +85,8 @@ class FullText(Field):
                 column, operator, value, expression)
         return expression
 
+    @order_method
     def convert_order(self, name, tables, Model):
-        method = getattr(Model, 'order_%s' % name, None)
-        if method:
-            return method(tables)
         table, _ = tables[None]
         column = self.sql_column(table)
         column = self._domain_column('ilike', column)
