@@ -1,11 +1,12 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from collections import defaultdict, OrderedDict
-from threading import RLock
+import builtins
 import logging
+from collections import OrderedDict, defaultdict
+from threading import RLock
+
 from trytond.modules import load_modules, register_classes
 from trytond.transaction import Transaction
-import builtins
 
 __all__ = ['Pool', 'PoolMeta', 'PoolBase', 'isregisteredby']
 
@@ -83,11 +84,13 @@ class Pool(object):
         module = kwargs['module']
         type_ = kwargs['type_']
         depends = set(kwargs.get('depends', []))
-        assert type_ in ('model', 'report', 'wizard')
+        assert type_ in {'model', 'report', 'wizard'}, (
+            f"{type_} is not a valid type_")
         for cls in classes:
             mpool = Pool.classes[type_][module]
-            assert cls not in mpool, cls
-            assert issubclass(cls.__class__, PoolMeta), cls
+            assert cls not in mpool, f"{cls} is already registered"
+            assert issubclass(cls.__class__, PoolMeta), (
+                f"{cls} is missing metaclass {PoolMeta}")
             mpool[cls] = depends
 
     @staticmethod
@@ -102,7 +105,7 @@ class Pool(object):
         with cls._lock:
             for classes in Pool.classes.values():
                 classes.clear()
-            register_classes()
+            register_classes(with_test=cls.test)
             cls._started = True
 
     @classmethod
@@ -188,12 +191,13 @@ class Pool(object):
         except KeyError:
             if type == 'report':
                 from trytond.report import Report
+
                 # Keyword argument 'type' conflicts with builtin function
                 cls = builtins.type(name, (Report,), {'__slots__': ()})
                 cls.__setup__()
                 cls.__post_setup__()
                 self.add(cls, type)
-                self.setup_mixin(self._modules, type='report', name=name)
+                self.setup_mixin(type='report', name=name)
                 return self.get(name, type=type)
             raise
 
@@ -231,7 +235,8 @@ class Pool(object):
                         cls.__name__, (cls, previous_cls), {'__slots__': ()})
                 except KeyError:
                     pass
-                assert issubclass(cls, PoolBase), cls
+                assert issubclass(cls, PoolBase), (
+                    f"{cls} is not a subclass of {PoolBase}")
                 self.add(cls, type=type_)
                 classes[type_].append(cls)
         self._modules.append(module)
@@ -250,7 +255,7 @@ class Pool(object):
             for cls in lst:
                 cls.__post_setup__()
 
-    def setup_mixin(self, modules, type=None, name=None):
+    def setup_mixin(self, type=None, name=None):
         logger.info('setup mixin for "%s"', self.database_name)
         if type is not None:
             types = [type]
