@@ -1,23 +1,22 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import xmlrpc.client as client
 import datetime
 import logging
-
+import xmlrpc.client as client
 # convert decimal to float before marshalling:
 from decimal import Decimal
 
 import defusedxml.xmlrpc
-from werkzeug.wrappers import Response
 from werkzeug.exceptions import (
-    BadRequest, InternalServerError, Conflict, Forbidden, Locked,
+    BadRequest, Conflict, Forbidden, InternalServerError, Locked,
     TooManyRequests)
+from werkzeug.wrappers import Response
 
+from trytond.exceptions import (
+    ConcurrencyException, LoginException, MissingDependenciesException,
+    RateLimitException, TrytonException, UserWarning)
 from trytond.model.fields.dict import ImmutableDict
 from trytond.protocols.wrappers import Request
-from trytond.exceptions import (
-    TrytonException, UserWarning, LoginException, ConcurrencyException,
-    RateLimitException, MissingDependenciesException)
 from trytond.tools import cached_property
 
 logger = logging.getLogger(__name__)
@@ -26,10 +25,9 @@ defusedxml.xmlrpc.monkey_patch()
 
 
 def dump_decimal(self, value, write):
-    value = {'__class__': 'Decimal',
-        'decimal': str(value),
-        }
-    self.dump_struct(value, write)
+    write('<value><bigdecimal>')
+    write(str(Decimal(value)))
+    write('</bigdecimal></value>')
 
 
 def dump_date(self, value, write):
