@@ -1,23 +1,24 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import datetime
-import dateutil.tz
-import os
 import inspect
 import logging
 import math
+import operator
+import os
 import subprocess
 import tempfile
 import time
 import unicodedata
 import warnings
 import zipfile
-import operator
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
 from io import BytesIO
 from itertools import groupby
 
+import dateutil.tz
+
 try:
     import html2text
 except ImportError:
@@ -29,20 +30,31 @@ except ImportError:
     weasyprint = None
 
 from genshi.filters import Translator
+from genshi.template.text import TextTemplate
 
+from trytond.exceptions import UserError
 from trytond.i18n import gettext
 from trytond.pool import Pool, PoolBase
-from trytond.transaction import Transaction
-from trytond.tools import slugify
-from trytond.url import URLMixin
 from trytond.rpc import RPC
-from trytond.exceptions import UserError
+from trytond.tools import slugify
+from trytond.transaction import Transaction, check_access
+from trytond.url import URLMixin
+
+try:
+    from trytond.tools import barcode
+except ImportError:
+    barcode = None
+try:
+    from trytond.tools import qrcode
+except ImportError:
+    qrcode = None
 
 warnings.simplefilter("ignore")
 import relatorio.reporting  # noqa: E402
+
 warnings.resetwarnings()
 try:
-    from relatorio.templates.opendocument import Manifest, MANIFEST
+    from relatorio.templates.opendocument import MANIFEST, Manifest
 except ImportError:
     Manifest, MANIFEST = None, None
 
@@ -89,7 +101,15 @@ TIMEDELTA_DEFAULT_CONVERTER['w'] = TIMEDELTA_DEFAULT_CONVERTER['d'] * 7
 TIMEDELTA_DEFAULT_CONVERTER['M'] = TIMEDELTA_DEFAULT_CONVERTER['d'] * 30
 TIMEDELTA_DEFAULT_CONVERTER['Y'] = TIMEDELTA_DEFAULT_CONVERTER['d'] * 365
 
-REPORT_NAME_MAX_LENGTH = 200
+NO_BREAKING_SPACE = '\u00A0'
+
+# For most OS maximum filename is 255 but Excel has a limitation which include
+# the path of 218.
+# As on Windows report is most likely to be open from
+# C:\Users\<username>\AppData\Local\Temp\tryton_<random>\ which has a length of
+# 56 with 12 for username and 8 for random. So 162 should be the maximum but we
+# round it to 100.
+REPORT_NAME_MAX_LENGTH = 100
 
 
 class TranslateFactory:
@@ -139,12 +159,14 @@ class Report(URLMixin, PoolBase):
             report type,
             data,
             a boolean to direct print,
-            the report name
+            the report name (with or without the record names)
         '''
         pool = Pool()
         ActionReport = pool.get('ir.action.report')
         ModelAccess = pool.get('ir.model.access')
         cls.check_access()
+        context = Transaction().context
+        ids = list(map(int, ids))
 
         action_id = data.get('action_id')
         if action_id is None:
@@ -163,8 +185,15 @@ class Report(URLMixin, PoolBase):
             max_length = (REPORT_NAME_MAX_LENGTH
                 - reserved_length
                 - len(str(record_count)) - 2)
+            if action_report.record_name:
+                template = TextTemplate(action_report.record_name)
+            else:
+                template = None
             for record in records[:5]:
-                record_name = record.rec_name
+                if template:
+                    record_name = template.generate(record=record).render()
+                else:
+                    record_name = record.rec_name
                 name_length += len(
                     unicodedata.normalize('NFKD', record_name)) + 1
                 if name_length > max_length:
@@ -181,7 +210,7 @@ class Report(URLMixin, PoolBase):
         if model:
             records = cls._get_records(ids, model, data)
 
-        with Transaction().set_context(_check_access=True):
+        with check_access():
             if model:
                 Model = pool.get(model)
                 ModelAccess.check(model, 'read')
@@ -225,11 +254,14 @@ class Report(URLMixin, PoolBase):
         if not isinstance(content, str):
             content = bytearray(content) if bytes == str else bytes(content)
         action_report_name = action_report.name[:REPORT_NAME_MAX_LENGTH]
-        filename = join_string.join(
-            filter(None, [
-                action_report_name,
-                report_name(
-                    records, len(action_report_name) + len(join_string))]))
+        if context.get('with_rec_name', True):
+            filename = join_string.join(
+                filter(None, [
+                    action_report_name,
+                    report_name(
+                        records, len(action_report_name) + len(join_string))]))
+        else:
+            filename = action_report_name
         return (oext, content, action_report.direct_print, filename)
 
     @classmethod
@@ -250,6 +282,7 @@ class Report(URLMixin, PoolBase):
 
         class TranslateModel(object):
             _languages = {}
+            __class__ = Model
 
             def __init__(self, id):
                 self.id = id
@@ -300,7 +333,12 @@ class Report(URLMixin, PoolBase):
         report_context['format_timedelta'] = cls.format_timedelta
         report_context['format_currency'] = cls.format_currency
         report_context['format_number'] = cls.format_number
+        report_context['format_number_symbol'] = cls.format_number_symbol
         report_context['datetime'] = datetime
+        if barcode:
+            report_context['barcode'] = cls.barcode
+        if qrcode:
+            report_context['qrcode'] = cls.qrcode
 
         def set_lang(language=None):
             if isinstance(language, Lang):
@@ -455,7 +493,7 @@ class Report(URLMixin, PoolBase):
                 # Add space if no time
                 text += ' '
             text += ('%.6f' % value)[1:]
-        return text
+        return text.replace(' ', NO_BREAKING_SPACE)
 
     @classmethod
     def format_currency(
@@ -465,17 +503,40 @@ class Report(URLMixin, PoolBase):
         Lang = pool.get('ir.lang')
         if lang is None:
             lang = Lang.get()
-        return lang.currency(value, currency, symbol, grouping, digits=digits)
+        return lang.currency(
+            value, currency, symbol=symbol, grouping=grouping, digits=digits)
 
     @classmethod
-    def format_number(cls, value, lang, digits=2, grouping=True,
-            monetary=None):
+    def format_number(
+            cls, value, lang, digits=None, grouping=True, monetary=None):
         pool = Pool()
         Lang = pool.get('ir.lang')
         if lang is None:
             lang = Lang.get()
-        return lang.format('%.' + str(digits) + 'f', value,
-            grouping=grouping, monetary=monetary)
+        return lang.format_number(
+            value, digits=digits, grouping=grouping, monetary=monetary)
+
+    @classmethod
+    def format_number_symbol(
+            cls, value, lang, symbol, digits=None, grouping=True):
+        pool = Pool()
+        Lang = pool.get('ir.lang')
+        if lang is None:
+            lang = Lang.get()
+        return lang.format_number_symbol(
+            value, symbol, digits=digits, grouping=grouping)
+
+    if barcode:
+        @classmethod
+        def barcode(cls, name, code, size=(), **kwargs):
+            image = barcode.generate_svg(name, code, **kwargs)
+            return (image, 'image/svg+xml', *size)
+
+    if qrcode:
+        @classmethod
+        def qrcode(cls, code, size=(), **kwargs):
+            image = qrcode.generate_svg(code, **kwargs)
+            return (image, 'image/svg+xml', *size)
 
 
 def get_email(report, record, languages):
@@ -497,7 +558,8 @@ def get_email(report, record, languages):
     msg = MIMEMultipart('alternative')
     msg.add_header('Content-Language', ', '.join(l.code for l in languages))
     for language in languages:
-        with Transaction().set_context(language=language.code):
+        with Transaction().set_context(
+                language=language.code, with_rec_name=False):
             ext, content, _, title = Report_.execute(
                 [record.id], {
                     'action_id': report_id,
