@@ -1,21 +1,9 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from trytond.model import (ModelSingleton, ModelSQL, UnionMixin, fields,
-    sequence_ordered)
-from trytond.transaction import Transaction
-
-__all__ = [
-    'Model',
-    'ModelParent', 'ModelChild', 'ModelChildChild',
-    'Singleton', 'URLObject',
-    'ModelStorage', 'ModelStorageRequired', 'ModelStorageContext',
-    'ModelSQLRequiredField', 'ModelSQLTimestamp', 'ModelSQLFieldSet',
-    'Model4Union1', 'Model4Union2', 'Model4Union3', 'Model4Union4',
-    'Union', 'UnionUnion',
-    'Model4UnionTree1', 'Model4UnionTree2', 'UnionTree',
-    'SequenceOrderedModel',
-    'NullOrder',
-    ]
+from trytond.model import (
+    ModelSingleton, ModelSQL, UnionMixin, fields, sequence_ordered)
+from trytond.pool import Pool
+from trytond.pyson import Eval
 
 
 class Model(ModelSQL):
@@ -28,6 +16,7 @@ class ModelParent(Model):
     "Model Parent"
     __name__ = 'test.model_parent'
     name = fields.Char("Name")
+    children = fields.One2Many('test.model_child', 'parent', "Children")
 
 
 class ModelChild(Model):
@@ -44,6 +33,41 @@ class ModelChildChild(Model):
     parent = fields.Many2One('test.model_child', "Parent")
 
 
+class ModelContext(Model):
+    "Model with contextual field"
+    __name__ = 'test.model_context'
+    name = fields.Char("Name")
+    target = fields.Many2One(
+        'test.model', "Target",
+        context={
+            'name': Eval('name'),
+            })
+
+
+class ModelContextParent(Model):
+    "Model with contextual field from _parent"
+    __name__ = 'test.model_context_parent'
+    parent = fields.Many2One('test.model', "Parent")
+    target = fields.Many2One(
+        'test.model', "Target",
+        context={
+            'name': Eval('_parent_parent', {}).get('name'),
+            })
+
+
+class ModelDefault(Model):
+    "Model with default"
+    __name__ = 'test.model.default'
+
+    name = fields.Char("Name")
+    description = fields.Char("Description")
+    target = fields.Many2One('test.model', "Target")
+
+    @classmethod
+    def default_description(cls):
+        return "Test"
+
+
 class Singleton(ModelSingleton, ModelSQL):
     'Singleton'
     __name__ = 'test.singleton'
@@ -60,55 +84,6 @@ class URLObject(ModelSQL):
     name = fields.Char('Name')
 
 
-class ModelStorage(ModelSQL):
-    'Model stored'
-    __name__ = 'test.modelstorage'
-    name = fields.Char('Name')
-
-
-class ModelStorageRequired(ModelSQL):
-    'Model stored'
-    __name__ = 'test.modelstorage.required'
-    name = fields.Char('Name', required=True)
-
-
-class ModelStorageContext(ModelSQL):
-    'Model Storage to test Context'
-    __name__ = 'test.modelstorage.context'
-    context = fields.Function(fields.Binary('Context'), 'get_context')
-
-    def get_context(self, name):
-        return Transaction().context
-
-
-class ModelSQLRequiredField(ModelSQL):
-    'model with a required field'
-    __name__ = 'test.modelsql'
-
-    integer = fields.Integer(string="integer", required=True)
-    desc = fields.Char(string="desc", required=True)
-
-
-class ModelSQLTimestamp(ModelSQL):
-    'Model to test timestamp'
-    __name__ = 'test.modelsql.timestamp'
-
-
-class ModelSQLFieldSet(ModelSQL):
-    'Model to test field set'
-    __name__ = 'test.modelsql.field_set'
-
-    field = fields.Function(fields.Integer('Field'),
-        'get_field', setter='set_field')
-
-    def get_field(self, name=None):
-        return
-
-    @classmethod
-    def set_field(cls, records, name, value):
-        pass
-
-
 class Model4Union1(ModelSQL):
     'Model for union 1'
     __name__ = 'test.model.union1'
@@ -185,7 +160,25 @@ class SequenceOrderedModel(sequence_ordered(), ModelSQL):
     __name__ = 'test.order.sequence'
 
 
-class NullOrder(ModelSQL):
-    "Null Order"
-    __name__ = 'test.null_order'
-    integer = fields.Integer('Integer')
+def register(module):
+    Pool.register(
+        Model,
+        ModelParent,
+        ModelChild,
+        ModelChildChild,
+        ModelContext,
+        ModelContextParent,
+        ModelDefault,
+        Singleton,
+        URLObject,
+        Model4Union1,
+        Model4Union2,
+        Model4Union3,
+        Model4Union4,
+        Union,
+        UnionUnion,
+        Model4UnionTree1,
+        Model4UnionTree2,
+        UnionTree,
+        SequenceOrderedModel,
+        module=module, type_='model')
