@@ -3,8 +3,8 @@
 
 import unittest
 
-from trytond.tests.test_tryton import activate_module, with_transaction
 from trytond.pool import Pool
+from trytond.tests.test_tryton import activate_module, with_transaction
 
 
 class FieldContextTestCase(unittest.TestCase):
@@ -24,15 +24,9 @@ class FieldContextTestCase(unittest.TestCase):
         parent = Parent(name='foo', child=child)
         parent.save()
         self.assertEqual(parent.child._context['name'], 'foo')
+        self.assertEqual(parent.child._context['rec_name'], '')
 
         parent.name = 'bar'
         parent.save()
         self.assertEqual(parent.child._context['name'], 'bar')
-
-
-def suite():
-    func = unittest.TestLoader().loadTestsFromTestCase
-    suite = unittest.TestSuite()
-    for testcase in (FieldContextTestCase,):
-        suite.addTests(func(testcase))
-    return suite
+        self.assertEqual(parent.child._context['rec_name'], '')
