@@ -5,7 +5,7 @@ import unittest
 from sql import Literal
 
 from trytond.model.exceptions import (
-    RequiredValidationError, DomainValidationError)
+    DomainValidationError, RequiredValidationError)
 from trytond.pool import Pool
 from trytond.tests.test_tryton import activate_module, with_transaction
 
@@ -58,6 +58,16 @@ class FieldIntegerTestCase(unittest.TestCase):
 
         self.assertEqual(integer.integer, 5)
 
+    @with_transaction()
+    def test_set_sql_value(self):
+        "Test cannot set SQL value"
+        Integer = Pool().get('test.integer_default')
+
+        integer = Integer()
+
+        with self.assertRaises(ValueError):
+            integer.integer = Literal(42)
+
     @with_transaction()
     def test_create_non_integer(self):
         "Test create integer with non integer"
@@ -103,7 +113,10 @@ class FieldIntegerTestCase(unittest.TestCase):
         "Test create integer with domain invalid"
         Integer = Pool().get('test.integer_domain')
 
-        with self.assertRaises(DomainValidationError):
+        with self.assertRaisesRegex(
+                DomainValidationError,
+                'The value "10" for field "Integer" '
+                'in ".*" of "Integer Domain"'):
             Integer.create([{
                         'integer': 10,
                         }])
@@ -334,7 +347,3 @@ class FieldIntegerTestCase(unittest.TestCase):
             Integer.write([integer], {
                     'integer': 'non integer',
                     })
-
-
-def suite():
-    return unittest.TestLoader().loadTestsFromTestCase(FieldIntegerTestCase)
