@@ -3,7 +3,7 @@
 import unittest
 
 from trytond.model.exceptions import (
-    RequiredValidationError, SizeValidationError)
+    DomainValidationError, RequiredValidationError, SizeValidationError)
 from trytond.pool import Pool
 from trytond.tests.test_tryton import activate_module, with_transaction
 
@@ -435,6 +435,36 @@ class FieldMany2ManyTestCase(unittest.TestCase, CommonTestCaseMixin):
                             ],
                         }])
 
+    @with_transaction()
+    def test_create_with_domain_valid(self):
+        "Test create many2many with valid domain"
+        pool = Pool()
+        Many2Many = pool.get('test.many2many_domain')
+
+        many2many, = Many2Many.create([{
+                    'targets': [
+                        ('create', [{'value': 42}]),
+                        ],
+                    }])
+
+        self.assertEqual(len(many2many.targets), 1)
+
+    @with_transaction()
+    def test_create_with_domain_invalid(self):
+        "Test create many2many with invalid domain"
+        pool = Pool()
+        Many2Many = pool.get('test.many2many_domain')
+
+        with self.assertRaisesRegex(
+                DomainValidationError,
+                'The value ".*,.*" for field "Targets" '
+                'in ".*" of "Many2Many Domain"'):
+            Many2Many.create([{
+                        'targets': [
+                            ('create', [{'value': 10}, {'value': None}]),
+                            ],
+                        }])
+
     @with_transaction()
     def test_create_filter(self):
         "Test create many2many with filter"
@@ -739,12 +769,3 @@ class FieldMany2ManyReferenceTestCase(unittest.TestCase, CommonTestCaseMixin):
 
     def Many2ManyTarget(self):
         return Pool().get('test.many2many_reference.target')
-
-
-def suite():
-    suite_ = unittest.TestSuite()
-    loader = unittest.TestLoader()
-    suite_.addTests(loader.loadTestsFromTestCase(FieldMany2ManyTestCase))
-    suite_.addTests(
-        loader.loadTestsFromTestCase(FieldMany2ManyReferenceTestCase))
-    return suite_
