@@ -243,8 +243,10 @@ class CommonTestCaseMixin:
     def test_search_ilike(self):
         "Test search text ilike"
         Text = self.Text()
-        text, = Text.create([{
+        text, sambreville = Text.create([{
                     'text': "Bar",
+                    }, {
+                    'text': "Sambreville",
                     }])
         with Transaction().set_context({
                     '%s.text.search_full_text' % Text.__name__: False,
@@ -267,6 +269,10 @@ class CommonTestCaseMixin:
         self.assertListEqual(texts_foo, [])
         self.assertListEqual(texts_f, [])
 
+        self.assertListEqual(
+            Text.search([('text', 'ilike', "%Sambreville%")]),
+            [sambreville])
+
     @unittest.skipIf(backend.name == 'sqlite',
         "SQLite does not have full text search")
     @with_transaction()
@@ -647,12 +653,3 @@ class FieldFullTextTestCase(unittest.TestCase):
             self.assertListEqual(
                 Model.search([], order=[('full_text', 'DESC')]),
                 [fat_cat, sad_cat, dog])
-
-
-def suite():
-    suite_ = unittest.TestSuite()
-    loader = unittest.TestLoader()
-    suite_.addTests(loader.loadTestsFromTestCase(FieldTextTestCase))
-    suite_.addTests(loader.loadTestsFromTestCase(FieldTextTranslatedTestCase))
-    suite_.addTests(loader.loadTestsFromTestCase(FieldFullTextTestCase))
-    return suite_
