@@ -5,10 +5,11 @@ import unittest
 
 from trytond.model import EvalEnvironment
 from trytond.model.exceptions import (
-    RequiredValidationError, DomainValidationError)
+    AccessError, DomainValidationError, RequiredValidationError)
+from trytond.model.modelstorage import _UnsavedRecordError
 from trytond.pool import Pool
-from trytond.transaction import Transaction
 from trytond.tests.test_tryton import activate_module, with_transaction
+from trytond.transaction import Transaction, check_access
 
 
 class ModelStorageTestCase(unittest.TestCase):
@@ -73,6 +74,21 @@ class ModelStorageTestCase(unittest.TestCase):
         count = ModelStorage.search_count([('name', '=', 'Test 5')])
         self.assertEqual(count, 1)
 
+    @with_transaction()
+    def test_browse_deleted(self):
+        "Test access record from browse list with deleted record"
+        pool = Pool()
+        ModelStorage = pool.get('test.modelstorage')
+        records = ModelStorage.create(
+            [{'name': 'Test %s' % i} for i in range(2)])
+
+        ModelStorage.delete(records[1:])
+        ModelStorage.write(records[:1], {})  # clean cache
+
+        records[0].name
+        with self.assertRaises(AccessError):
+            records[1].name
+
     @with_transaction()
     def test_browse_context(self):
         'Test context when browsing'
@@ -136,6 +152,78 @@ class ModelStorageTestCase(unittest.TestCase):
         with self.assertRaises(RequiredValidationError):
             ModelStorage.save([foo, bar])
 
+    @with_transaction()
+    def test_many2one_save_raise_exception(self):
+        "Test _save_values raises _UnsavedRecordError on unsaved record"
+        pool = Pool()
+        ModelStorage = pool.get('test.modelstorage.save_m2o')
+        Target = pool.get('test.modelstorage.save_m2o.target')
+
+        record = ModelStorage()
+        record.target = target = Target()
+
+        with self.assertRaises(_UnsavedRecordError) as cm:
+            record._save_values
+
+        self.assertEqual(cm.exception.record, target)
+
+    @with_transaction()
+    def test_save_many2one_create(self):
+        "Test saving a record while creating a Many2One"
+        pool = Pool()
+        ModelStorage = pool.get('test.modelstorage.save_m2o')
+        Target = pool.get('test.modelstorage.save_m2o.target')
+
+        record = ModelStorage()
+        record.target = target = Target()
+        with self.assertWarnsRegex(
+                UserWarning, r"Using an unsaved relation record:"):
+            record.save()
+
+        self.assertIsNotNone(record.id)
+        self.assertIsNotNone(target.id)
+
+    @with_transaction()
+    def test_save_many2one_create_multi(self):
+        "Test saving multiple records while creating a M2O"
+        pool = Pool()
+        ModelStorage = pool.get('test.modelstorage.save_m2o')
+        Target = pool.get('test.modelstorage.save_m2o.target')
+
+        record1 = ModelStorage()
+        record1.target = target = Target()
+        record1.save()
+
+        record2 = ModelStorage()
+        record2.target = target
+        record2.save()
+
+        ModelStorage.save([record1, record2])
+
+        self.assertIsNotNone(record1.id)
+        self.assertIsNotNone(record2.id)
+        self.assertEqual(record1.target.id, record2.target.id)
+        self.assertNotEqual(record2.id, record1.id)
+
+    @with_transaction()
+    def test_save_many2one_in_one2many_create(self):
+        "Test saving a record while creating a M2O contained in a O2M"
+        pool = Pool()
+        ModelStorage = pool.get('test.modelstorage.save_m2o')
+        Target = pool.get('test.modelstorage.save_m2o.target')
+        M2OTarget = pool.get('test.modelstorage.save_m2o.o2m_target.target')
+
+        target = Target()
+        target.target = m2o_target = M2OTarget()
+        record = ModelStorage()
+        record.targets = [target]
+        record.save()
+
+        self.assertIsNotNone(record.id)
+        self.assertIsNotNone(m2o_target.id)
+        self.assertEqual(len(record.targets), 1)
+        self.assertEqual(record.targets[0].target, m2o_target)
+
     @with_transaction()
     def test_save_one2many_create(self):
         "Test save one2many create"
@@ -423,6 +511,57 @@ class ModelStorageTestCase(unittest.TestCase):
         self.assertEqual(cm.exception.domain[0], [['value', '!=', 'valid']])
         self.assertTrue(cm.exception.domain[1], 'value')
 
+    @with_transaction()
+    def test_relation_domain_context(self):
+        "Test valid relation with context"
+        pool = Pool()
+        Model = pool.get('test.modelstorage.relation_domain')
+        Target = pool.get('test.modelstorage.relation_domain.target')
+
+        target, = Target.create([{}])
+
+        record, = Model.create(
+            [{'relation_context': target.id, 'relation_valid': True}])
+
+    @with_transaction()
+    def test_relation_domain_context_invalid(self):
+        "Test invalid relation with context"
+        pool = Pool()
+        Model = pool.get('test.modelstorage.relation_domain')
+        Target = pool.get('test.modelstorage.relation_domain.target')
+
+        target, = Target.create([{}])
+
+        with self.assertRaises(DomainValidationError):
+            record, = Model.create(
+                [{'relation_context': target.id, 'relation_valid': False}])
+
+    @with_transaction()
+    def test_relation_pyson_domain_context(self):
+        "Test valid relation with PYSON and context"
+        pool = Pool()
+        Model = pool.get('test.modelstorage.relation_domain')
+        Target = pool.get('test.modelstorage.relation_domain.target')
+
+        target, = Target.create([{}])
+
+        record, = Model.create(
+            [{'relation_pyson_context': target.id, 'relation_valid': True}])
+
+    @with_transaction()
+    def test_relation_pyson_domain_context_invalid(self):
+        "Test invalid relation with PYSON and context"
+        pool = Pool()
+        Model = pool.get('test.modelstorage.relation_domain')
+        Target = pool.get('test.modelstorage.relation_domain.target')
+
+        target, = Target.create([{}])
+
+        with self.assertRaises(DomainValidationError):
+            record, = Model.create([{
+                        'relation_pyson_context': target.id,
+                        'relation_valid': False}])
+
     @with_transaction()
     def test_relation2_domain_invalid(self):
         "Test invalid relation domain with 2 level"
@@ -442,6 +581,22 @@ class ModelStorageTestCase(unittest.TestCase):
         self.assertTrue(
             cm.exception.domain[1]['relation2']['relation_fields']['value'])
 
+    @with_transaction()
+    def test_domain_not_required_validation(self):
+        "Test domain validation on not required"
+        pool = Pool()
+        Model = pool.get('test.modelstorage.domain_not_required')
+        Model.create([{'domain_not_required': None}])
+
+    @with_transaction()
+    def test_domain_not_required_invalid(self):
+        "Test invalid domain on not required"
+        pool = Pool()
+        Model = pool.get('test.modelstorage.domain_not_required')
+
+        with self.assertRaises(DomainValidationError):
+            Model.create([{'domain_not_required': 0}])
+
     @with_transaction()
     def test_check_xml_record_without_record(self):
         "Test check_xml_record without record"
@@ -449,9 +604,7 @@ class ModelStorageTestCase(unittest.TestCase):
         Model = pool.get('test.modelstorage')
         record, = Model.create([{}])
 
-        result = Model.check_xml_record([record], {'name': "Test"})
-
-        self.assertTrue(result)
+        Model.check_xml_record([record], {'name': "Test"})
 
     @with_transaction()
     def test_check_xml_record_with_record_no_matching_values(self):
@@ -469,9 +622,7 @@ class ModelStorageTestCase(unittest.TestCase):
                     'noupdate': False,
                     }])
 
-        result = Model.check_xml_record([record], {'name': "Test"})
-
-        self.assertTrue(result)
+        Model.check_xml_record([record], {'name': "Test"})
 
     @with_transaction()
     def test_check_xml_record_with_record_matching_values(self):
@@ -489,9 +640,8 @@ class ModelStorageTestCase(unittest.TestCase):
                     'noupdate': False,
                     }])
 
-        result = Model.check_xml_record([record], {'name': "Bar"})
-
-        self.assertFalse(result)
+        with self.assertRaises(AccessError):
+            Model.check_xml_record([record], {'name': "Bar"})
 
     @with_transaction()
     def test_check_xml_record_with_record_matching_values_noupdate(self):
@@ -509,9 +659,7 @@ class ModelStorageTestCase(unittest.TestCase):
                     'noupdate': True,
                     }])
 
-        result = Model.check_xml_record([record], {'name': "Bar"})
-
-        self.assertTrue(result)
+        Model.check_xml_record([record], {'name': "Bar"})
 
     @with_transaction(user=0)
     def test_check_xml_record_with_record_matching_values_root(self):
@@ -529,9 +677,7 @@ class ModelStorageTestCase(unittest.TestCase):
                     'noupdate': False,
                     }])
 
-        result = Model.check_xml_record([record], {'name': "Bar"})
-
-        self.assertTrue(result)
+        Model.check_xml_record([record], {'name': "Bar"})
 
     @with_transaction()
     def test_check_xml_record_with_record_no_values(self):
@@ -549,9 +695,8 @@ class ModelStorageTestCase(unittest.TestCase):
                     'noupdate': False,
                     }])
 
-        result = Model.check_xml_record([record], None)
-
-        self.assertFalse(result)
+        with self.assertRaises(AccessError):
+            Model.check_xml_record([record], None)
 
     @with_transaction()
     def test_check_xml_record_with_record_no_values_noupdate(self):
@@ -569,9 +714,7 @@ class ModelStorageTestCase(unittest.TestCase):
                     'noupdate': True,
                     }])
 
-        result = Model.check_xml_record([record], None)
-
-        self.assertTrue(result)
+        Model.check_xml_record([record], None)
 
     @with_transaction()
     def test_delete_clear_db_id_model_data_noupdate(self):
@@ -702,7 +845,7 @@ class EvalEnvironmentTestCase(unittest.TestCase):
                     'perm_write': False,
                     'perm_delete': False,
                     }])
-        with Transaction().set_context(_check_access=True):
+        with check_access():
             record = Model(name="Test")
             record.save()
 
@@ -724,15 +867,7 @@ class EvalEnvironmentTestCase(unittest.TestCase):
                     }])
         record, = Model.create([{'name': "Test"}])
 
-        with Transaction().set_context(_check_access=True):
+        with check_access():
             record = Model(record.id)
 
             self.assertEqual(record.name, "Test")
-
-
-def suite():
-    suite_ = unittest.TestSuite()
-    loader = unittest.TestLoader()
-    suite_.addTests(loader.loadTestsFromTestCase(ModelStorageTestCase))
-    suite_.addTests(loader.loadTestsFromTestCase(EvalEnvironmentTestCase))
-    return suite_
