@@ -3,22 +3,37 @@
 # this repository contains the full copyright notices and license terms.
 
 import datetime as dt
-import unittest
 import doctest
 import sys
+import unittest
+from copy import deepcopy
+from io import BytesIO
 
 import sql
 import sql.operators
 
 from trytond.tools import (
-    reduce_ids, reduce_domain, decimal_, is_instance_method, file_open,
-    strip_wildcard, lstrip_wildcard, rstrip_wildcard, slugify, sortable_values,
-    escape_wildcard, unescape_wildcard, is_full_text, firstline)
-from trytond.tools.string_ import StringPartitioned, LazyString
+    decimal_, escape_wildcard, file_open, firstline, grouped_slice,
+    is_full_text, is_instance_method, likify, lstrip_wildcard,
+    pairwise_longest, reduce_domain, reduce_ids, remove_forbidden_chars,
+    rstrip_wildcard, slugify, sortable_values, strip_wildcard, timezone,
+    unescape_wildcard)
 from trytond.tools.domain_inversion import (
-    domain_inversion, parse, simplify, merge, concat, unique_value,
-    eval_domain, localize_domain,
-    prepare_reference_domain, extract_reference_models)
+    canonicalize, concat, domain_inversion, eval_domain,
+    extract_reference_models, localize_domain, merge, parse,
+    prepare_reference_domain, simplify, sort, unique_value)
+from trytond.tools.immutabledict import ImmutableDict
+from trytond.tools.logging import format_args
+from trytond.tools.string_ import LazyString, StringPartitioned
+
+try:
+    from trytond.tools import barcode
+except ImportError:
+    barcode = None
+try:
+    from trytond.tools import qrcode
+except ImportError:
+    qrcode = None
 
 
 class ToolsTestCase(unittest.TestCase):
@@ -98,6 +113,32 @@ class ToolsTestCase(unittest.TestCase):
             self.assertEqual(reduce_domain(i), j,
                     '%s -> %s != %s' % (i, reduce_domain(i), j))
 
+    def test_grouped_slice(self):
+        "Test grouped slice"
+        for (values, count, result) in [
+                (list(range(10)), 5, [[0, 1, 2, 3, 4], [5, 6, 7, 8, 9]]),
+                (list(range(5)), 5, [[0, 1, 2, 3, 4]]),
+                (list(range(5)), 2, [[0, 1], [2, 3], [4]]),
+                ]:
+            with self.subTest(values=values, count=count):
+                self.assertEqual(
+                    list(map(list, grouped_slice(values, count))), result)
+
+    def test_grouped_slice_generator(self):
+        "Test grouped slice"
+        self.assertEqual(
+            list(map(list, grouped_slice((x for x in range(10)), 5))),
+            [[0, 1, 2, 3, 4], [5, 6, 7, 8, 9]])
+
+    def test_pairwise_longest(self):
+        "Test pairwise_longest"
+        self.assertEqual(
+            list(pairwise_longest(range(3))), [(0, 1), (1, 2), (2, None)])
+
+    def test_pairwise_longest_empty(self):
+        "Test pairwise_longest empty"
+        self.assertEqual(list(pairwise_longest([])), [])
+
     def test_is_instance_method(self):
         'Test is_instance_method'
 
@@ -126,7 +167,8 @@ class ToolsTestCase(unittest.TestCase):
         with file_open('ir/__init__.py') as fp:
             self.assertTrue(fp)
 
-        with self.assertRaisesRegex(IOError, "File not found :"):
+        with self.assertRaisesRegex(
+                FileNotFoundError, "No such file or directory:"):
             with file_open('ir/noexist'):
                 pass
 
@@ -147,26 +189,26 @@ class ToolsTestCase(unittest.TestCase):
         'Test strip wildcard'
         for clause, result in [
                 ('%a%', 'a'),
-                ('%%%%a%%%', 'a'),
+                ('%%a%%', '%a%'),
                 ('\\%a%', '\\%a'),
                 ('\\%a\\%', '\\%a\\%'),
                 ('a', 'a'),
                 ('', ''),
                 (None, None),
                 ]:
-            self.assertEqual(
-                strip_wildcard(clause), result, msg=clause)
+            with self.subTest(clause=clause):
+                self.assertEqual(strip_wildcard(clause), result)
 
     def test_strip_wildcard_different_wildcard(self):
         'Test strip wildcard with different wildcard'
-        self.assertEqual(strip_wildcard('___a___', '_'), 'a')
+        self.assertEqual(strip_wildcard('_a_', '_'), 'a')
 
     def test_lstrip_wildcard(self):
         'Test lstrip wildcard'
         for clause, result in [
                 ('%a', 'a'),
                 ('%a%', 'a%'),
-                ('%%%%a%', 'a%'),
+                ('%%a%', '%a%'),
                 ('\\%a%', '\\%a%'),
                 ('a', 'a'),
                 ('', ''),
@@ -177,14 +219,14 @@ class ToolsTestCase(unittest.TestCase):
 
     def test_lstrip_wildcard_different_wildcard(self):
         'Test lstrip wildcard with different wildcard'
-        self.assertEqual(lstrip_wildcard('___a', '_'), 'a')
+        self.assertEqual(lstrip_wildcard('_a', '_'), 'a')
 
     def test_rstrip_wildcard(self):
         'Test rstrip wildcard'
         for clause, result in [
                 ('a%', 'a'),
                 ('%a%', '%a'),
-                ('%a%%%%%', '%a'),
+                ('%a%%', '%a%'),
                 ('%a\\%', '%a\\%'),
                 ('a', 'a'),
                 ('', ''),
@@ -193,8 +235,8 @@ class ToolsTestCase(unittest.TestCase):
             self.assertEqual(
                 rstrip_wildcard(clause), result, msg=clause)
 
-    def test_rstrip_wildcard_diferent_wildcard(self):
-        self.assertEqual(rstrip_wildcard('a___', '_'), 'a')
+    def test_rstrip_wildcard_different_wildcard(self):
+        self.assertEqual(rstrip_wildcard('a_', '_'), 'a')
 
     def test_escape_wildcard(self):
         self.assertEqual(
@@ -212,6 +254,8 @@ class ToolsTestCase(unittest.TestCase):
         for value, result in [
                 ('foo', True),
                 ('%foo bar%', True),
+                ('%%foo bar%', False),
+                ('%foo bar%%', False),
                 ('foo%', False),
                 ('foo_bar', False),
                 ('foo\\_bar', True),
@@ -219,6 +263,19 @@ class ToolsTestCase(unittest.TestCase):
             with self.subTest(value=value):
                 self.assertEqual(is_full_text(value), result)
 
+    def test_likify(self):
+        "Test likify"
+        for value, result in [
+                ('', '%'),
+                ('foo', '%foo%'),
+                ('foo%', 'foo%'),
+                ('f_o', 'f_o'),
+                ('%foo%', '%foo%'),
+                ('foo\\_bar', '%foo\\_bar%'),
+                ]:
+            with self.subTest(value=value):
+                self.assertEqual(likify(value), result)
+
     def test_slugify(self):
         "Test slugify"
         self.assertEqual(slugify('unicode ♥ is ☢'), 'unicode-is')
@@ -227,6 +284,128 @@ class ToolsTestCase(unittest.TestCase):
         "Test hyphenate in slugify"
         self.assertEqual(slugify('foo bar', hyphenate='_'), 'foo_bar')
 
+    def test_sortable_values(self):
+        def key(values):
+            return values
+
+        values = [
+            (('a', 1), ('b', None)),
+            (('a', 1), ('b', 3)),
+            (('a', 1), ('b', 2)),
+            ]
+
+        with self.assertRaises(TypeError):
+            sorted(values, key=key)
+        self.assertEqual(
+            sorted(values, key=sortable_values(key)), [
+                (('a', 1), ('b', 2)),
+                (('a', 1), ('b', 3)),
+                (('a', 1), ('b', None)),
+                ])
+
+    def test_firstline(self):
+        "Test firstline"
+        for text, result in [
+                ("", ""),
+                ("first line\nsecond line", "first line"),
+                ("\nsecond line", "second line"),
+                ("\n\nthird line", "third line"),
+                (" \nsecond line", "second line"),
+                ]:
+            with self.subTest(text=text, result=result):
+                self.assertEqual(firstline(text), result)
+
+    def test_remove_forbidden_chars(self):
+        "Test remove_forbidden_chars"
+        for string, result in [
+                ("", ""),
+                (None, None),
+                ("\ttest", "test"),
+                (" test ", "test"),
+                ]:
+            with self.subTest(string=string):
+                self.assertEqual(remove_forbidden_chars(string), result)
+
+    def test_get_tzinfo_valid(self):
+        "Test get_tzinfo with an valid timezone"
+        zi = timezone.get_tzinfo('Europe/Brussels')
+        now = dt.datetime(2022, 5, 17, tzinfo=zi)
+        self.assertEqual(str(now), "2022-05-17 00:00:00+02:00")
+
+    def test_get_tzinfo_invalid(self):
+        "Test get_tzinfo with an invalid timezone"
+        zi = timezone.get_tzinfo('foo')
+        now = dt.datetime(2022, 5, 17, tzinfo=zi)
+        self.assertEqual(str(now), "2022-05-17 00:00:00+00:00")
+
+    def test_availabe_timezones(self):
+        "Test available_timezones"
+        available_timezones = timezone.available_timezones()
+        self.assertTrue(available_timezones)
+        self.assertIsInstance(available_timezones, set)
+
+    def test_format_args(self):
+        "Test format_args"
+        for args, kwargs, short_form, long_form in [
+                (tuple(), {}, "()", "()"),
+                (('abcdefghijklmnopqrstuvwxyz',), {},
+                    "('abcdefghijklmnopq...')",
+                    "('abcdefghijklmnopqrstuvwxyz')"),
+                ((b'foo',), {}, "(<3 bytes>)", "(b'foo')"),
+                (([1, 2, 3, 4], [4, 5, 6, 7], [8, 9, 10, 11]), {},
+                    "([1, 2, 3, 4], [4, 5, 6, 7], [8, 9, 10, 11])",
+                    "([1, 2, 3, 4], [4, 5, 6, 7], [8, 9, 10, 11])"),
+                (([1, 2, 3, 4, 5, 6], [4, 5, 6, 7], [8, 9, 10, 11]), {},
+                    "([1, 2, 3, 4, 5, ...], [4, 5, 6, 7], [8, 9, 10, 11])",
+                    "([1, 2, 3, 4, 5, 6], [4, 5, 6, 7], [8, 9, 10, 11])"),
+                (([1, 2, 3], 'foo'), {'a': '1'},
+                    "([1, 2, 3], 'foo', a='1')",
+                    "([1, 2, 3], 'foo', a='1')"),
+                ((list(range(5)),), {},
+                    "([0, 1, 2, 3, 4])",
+                    "([0, 1, 2, 3, 4])"),
+                ((list(range(6)),), {},
+                    "([0, 1, 2, 3, 4, ...])",
+                    "([0, 1, 2, 3, 4, 5])"),
+                (('a', 'b', 'c', 'd'), {},
+                    "('a', 'b', 'c', ...)", "('a', 'b', 'c', 'd')"),
+                (([1, [2, [3, [4, [5, [6]]]]]],), {},
+                    "([1, [2, [3, [4, [5, ...]]]]])",
+                    "([1, [2, [3, [4, [5, [6]]]]]])"),
+                (tuple(), {'a': 1, 'b': 2}, "(a=1, b=2)", "(a=1, b=2)"),
+                ((list(range(10)), 'foo'), {'a': '1'},
+                    "([0, 1, 2, 3, 4, ...], 'foo', a='1')",
+                    "([0, 1, 2, 3, 4, 5, 6, 7, 8, 9], 'foo', a='1')"),
+                ((list(range(4)), 'foo'), {k: k for k in 'abcdefg'},
+                    "([0, 1, 2, 3], 'foo', a='a', ...)",
+                    "([0, 1, 2, 3], 'foo', a='a', b='b', c='c', d='d',"
+                    " e='e', f='f', g='g')"),
+                ((list(range(5)), list(range(20, 25))),
+                    {k: list(range(7)) for k in 'ab'},
+                    "([0, 1, 2, 3, 4], [20, 21, 22, 23, 24], "
+                    "a=[0, 1, 2, 3, 4, ...], ...)",
+                    "([0, 1, 2, 3, 4], [20, 21, 22, 23, 24], "
+                    "a=[0, 1, 2, 3, 4, 5, 6], b=[0, 1, 2, 3, 4, 5, 6])"),
+                (tuple(), {k: {i: i for i in range(7)} for k in 'abcd'},
+                    "(a={0: 0, 1: 1, 2: 2, 3: 3, 4: 4, ...}, "
+                    "b={0: 0, 1: 1, 2: 2, 3: 3, 4: 4, ...}, "
+                    "c={0: 0, 1: 1, 2: 2, 3: 3, 4: 4, ...}, ...)",
+                    "(a={0: 0, 1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6}, "
+                    "b={0: 0, 1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6}, "
+                    "c={0: 0, 1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6}, "
+                    "d={0: 0, 1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6})"),
+                ]:
+            with self.subTest(form='short', args=args, kwargs=kwargs):
+                self.assertEqual(
+                    str(format_args(args, kwargs, max_args=3, max_items=5)),
+                    short_form)
+            with self.subTest(form='long', args=args, kwargs=kwargs):
+                self.assertEqual(
+                    str(format_args(
+                            args, kwargs, verbose=True, max_args=3,
+                            max_items=5)),
+                    long_form)
+
 
 class StringPartitionedTestCase(unittest.TestCase):
     "Test StringPartitioned"
@@ -310,6 +489,77 @@ class LazyStringTestCase(unittest.TestCase):
         self.assertEqual(s, 'barfoo')
 
 
+class ImmutableDictTestCase(unittest.TestCase):
+    "Test ImmutableDict"
+
+    def test_setitem(self):
+        "__setitem__ not allowed"
+        d = ImmutableDict()
+
+        with self.assertRaises(TypeError):
+            d['foo'] = 'bar'
+
+    def test_delitem(self):
+        "__delitem__ not allowed"
+        d = ImmutableDict(foo='bar')
+
+        with self.assertRaises(TypeError):
+            del d['foo']
+
+    def test_ior(self):
+        "__ior__ not allowed"
+        d = ImmutableDict()
+
+        with self.assertRaises(TypeError):
+            d |= {'foo': 'bar'}
+
+    def test_clear(self):
+        "clear not allowed"
+        d = ImmutableDict(foo='bar')
+
+        with self.assertRaises(TypeError):
+            d.clear()
+
+    def test_pop(self):
+        "pop not allowed"
+        d = ImmutableDict(foo='bar')
+
+        with self.assertRaises(TypeError):
+            d.pop('foo')
+
+    def test_popitem(self):
+        "popitem not allowed"
+        d = ImmutableDict(foo='bar')
+
+        with self.assertRaises(TypeError):
+            d.popitem('foo')
+
+    def test_setdefault(self):
+        "setdefault not allowed"
+        d = ImmutableDict()
+
+        with self.assertRaises(TypeError):
+            d.setdefault('foo', 'bar')
+
+    def test_update(self):
+        "update not allowed"
+        d = ImmutableDict()
+
+        with self.assertRaises(TypeError):
+            d.update({'foo': 'bar'})
+
+    def test_deepcopy(self):
+        "deepcopying should be possible"
+        original = ImmutableDict(foo={'a': 1}, bar=2, baz=1.3)
+        copy = deepcopy(original)
+
+        self.assertEqual(original, copy)
+        self.assertIsNot(original, copy)
+
+        self.assertEqual(original['foo'], copy['foo'])
+        self.assertIsNot(original['foo'], copy['foo'])
+
+
 class DomainInversionTestCase(unittest.TestCase):
     "Test domain_inversion"
 
@@ -429,7 +679,7 @@ class DomainInversionTestCase(unittest.TestCase):
             domain_inversion(domain, 'x', {'y': 7, 'z': 'abc'}), True)
         self.assertEqual(
             domain_inversion(domain, 'x', {'y': 4, 'z': 'b'}),
-            ['OR', [['x', '=', 3]], [['x', '=', 2]]])
+            ['OR', ['x', '=', 3], ['x', '=', 2]])
 
     def test_parse(self):
         domain = parse([['x', '=', 5]])
@@ -451,16 +701,39 @@ class DomainInversionTestCase(unittest.TestCase):
         domain = [[['x', '=', 3]]]
         self.assertEqual(simplify(domain), [['x', '=', 3]])
 
+        domain = [[['x', '=', 3], ['y', '=', 4]]]
+        self.assertEqual(simplify(domain), [['x', '=', 3], ['y', '=', 4]])
+
         domain = ['OR', ['x', '=', 3]]
         self.assertEqual(simplify(domain), [['x', '=', 3]])
 
         domain = ['OR', [['x', '=', 3]], [['y', '=', 5]]]
         self.assertEqual(
-            simplify(domain), ['OR', [['x', '=', 3]], [['y', '=', 5]]])
+            simplify(domain), ['OR', ['x', '=', 3], ['y', '=', 5]])
 
         domain = ['OR', ['x', '=', 3], ['AND', ['y', '=', 5]]]
         self.assertEqual(
-            simplify(domain), ['OR', ['x', '=', 3], [['y', '=', 5]]])
+            simplify(domain), ['OR', ['x', '=', 3], ['y', '=', 5]])
+
+        domain = ['OR', ('x', '=', 1), ['OR', ('x', '=', 2), ('x', '=', 3)]]
+        self.assertEqual(
+            simplify(domain),
+            ['OR', ('x', '=', 1), ('x', '=', 2), ('x', '=', 3)])
+
+        domain = [['x', '=', 3], ['OR']]
+        self.assertEqual(simplify(domain), [['x', '=', 3]])
+
+        domain = ['OR', ['x', '=', 3], []]
+        self.assertEqual(simplify(domain), [])
+
+        domain = ['OR', ['x', '=', 3], ['OR']]
+        self.assertEqual(simplify(domain), [])
+
+        domain = [['x', '=', 3], []]
+        self.assertEqual(simplify(domain), [['x', '=', 3]])
+
+        domain = [['x', '=', 3], ['AND']]
+        self.assertEqual(simplify(domain), [['x', '=', 3]])
 
         domain = ['AND']
         self.assertEqual(simplify(domain), [])
@@ -468,6 +741,35 @@ class DomainInversionTestCase(unittest.TestCase):
         domain = ['OR']
         self.assertEqual(simplify(domain), [])
 
+    def test_simplify_deduplicate(self):
+        "Test deduplicate"
+        clause = ('x', '=', 'x')
+        another = ('y', '=', 'y')
+        third = ('z', '=', 'z')
+        tests = [
+            ([], []),
+            (['OR', []], []),
+            (['AND', []], []),
+            ([clause], [clause]),
+            (['OR', clause], [clause]),
+            ([clause, clause], [clause]),
+            (['OR', clause, clause], [clause]),
+            ([clause, [clause, clause]], [clause]),
+            ([clause, another], [clause, another]),
+            (['OR', clause, another], ['OR', clause, another]),
+            ([clause, clause, another], [clause, another]),
+            ([clause, [clause, clause], another], [clause, another]),
+            ([clause, clause, another, another], [clause, another]),
+            ([clause, another, clause, another], [clause, another]),
+            (
+                ['AND', ['OR', clause, another], third],
+                ['AND', ['OR', clause, another], third]),
+            ]
+
+        for input, expected in tests:
+            with self.subTest(input=input):
+                self.assertEqual(simplify(input), expected)
+
     def test_merge(self):
         domain = [['x', '=', 6], ['y', '=', 7]]
         self.assertEqual(merge(domain), ['AND', ['x', '=', 6], ['y', '=', 7]])
@@ -517,7 +819,7 @@ class DomainInversionTestCase(unittest.TestCase):
         self.assertEqual(concat([], []), [])
         self.assertEqual(
             concat(domain1, domain2, domoperator='OR'),
-            ['OR', [['a', '=', 1]], [['b', '=', 2]]])
+            ['OR', ['a', '=', 1], ['b', '=', 2]])
 
     def test_unique_value(self):
         domain = [['a', '=', 1]]
@@ -537,6 +839,27 @@ class DomainInversionTestCase(unittest.TestCase):
 
         domain = [['a.b.id', '=', 1, 'model']]
         self.assertEqual(unique_value(domain), (False, None, None))
+        self.assertEqual(
+            unique_value(domain, single_value=False), (False, None, None))
+
+        domain = [['a', 'in', [1]]]
+        self.assertEqual(unique_value(domain), (True, 'a', 1))
+        self.assertEqual(
+            unique_value(domain, single_value=False), (False, None, None))
+
+        domain = [['a', 'in', [1, 2]]]
+        self.assertEqual(unique_value(domain), (False, None, None))
+
+        domain = [['a', 'in', []]]
+        self.assertEqual(unique_value(domain), (False, None, None))
+
+        domain = [['a.b', 'in', [1]]]
+        self.assertEqual(unique_value(domain), (False, None, None))
+
+        domain = [['a.id', 'in', [1], 'model']]
+        self.assertEqual(unique_value(domain), (True, 'a.id', ['model', 1]))
+        self.assertEqual(
+            unique_value(domain, single_value=False), (False, None, None))
 
     def test_evaldomain(self):
         domain = [['x', '>', 5]]
@@ -817,44 +1140,96 @@ class DomainInversionTestCase(unittest.TestCase):
             extract_reference_models(domain, 'x'), {'model_A', 'model_B'})
         self.assertEqual(extract_reference_models(domain, 'y'), set())
 
-    def test_sortable_values(self):
-        def key(values):
-            return values
-
-        values = [
-            (('a', 1), ('b', None)),
-            (('a', 1), ('b', 3)),
-            (('a', 1), ('b', 2)),
-            ]
-
-        with self.assertRaises(TypeError):
-            sorted(values, key=key)
-        self.assertEqual(
-            sorted(values, key=sortable_values(key)), [
-                (('a', 1), ('b', 2)),
-                (('a', 1), ('b', 3)),
-                (('a', 1), ('b', None)),
-                ])
-
-    def test_firstline(self):
-        "Test firstline"
-        for text, result in [
-                ("", ""),
-                ("first line\nsecond line", "first line"),
-                ("\nsecond line", "second line"),
-                ("\n\nthird line", "third line"),
-                (" \nsecond line", "second line"),
+    def test_sort(self):
+        "Test sorting domain"
+        for (domain, expected) in [
+                ([], []),
+                (['AND'], ['AND']),
+                (['OR'], ['OR']),
+                ([('foo', '=', 1)], [('foo', '=', 1)]),
+                ([('foo', '=', 1000), ('foo', '=', 0)],
+                    [('foo', '=', 0), ('foo', '=', 1000)]),
+                ([('foo', '=', 1), ('foo', '=', None)],
+                    [('foo', '=', None), ('foo', '=', 1)]),
+                ([('foo', '=', 1), []], [[], ('foo', '=', 1)]),
+                ([[[[('foo', '=', 1)]]], [[('foo', '=', 1)]]],
+                    [[[('foo', '=', 1)]], [[[('foo', '=', 1)]]]]),
+                ([[('foo', '=', 1), ('bar', '=', 2)],
+                    [('foo', '=', 1), ('bar', '=', 2), ('baz', '=', 3)]],
+                    [[('bar', '=', 2), ('baz', '=', 3), ('foo', '=', 1)],
+                        [('bar', '=', 2), ('foo', '=', 1)]]),
+                (['OR', ('foo', '=', 1), ('foo', '=', None),
+                    ['AND', ('bar', '=', 2), ('baz', '=', 3)]],
+                    ['OR', ('foo', '=', None), ('foo', '=', 1),
+                        ['AND', ('bar', '=', 2), ('baz', '=', 3)]]),
+                ([('foo', '=', 'bar'), ('foo', '=', None)],
+                    [('foo', '=', None), ('foo', '=', 'bar')]),
                 ]:
-            with self.subTest(text=text, result=result):
-                self.assertEqual(firstline(text), result)
+            with self.subTest(domain=domain):
+                self.assertEqual(sort(domain), expected)
+
+    def test_canonicalize(self):
+        "Test domain canonicalization"
+        for (domain, expected) in [
+                ([], []),
+                (['AND'], []),
+                (['OR'], []),
+                ([('foo', '=', 1), ('bar', '=', 2)],
+                    [('bar', '=', 2), ('foo', '=', 1)]),
+                ([[('foo', '=', 1)]], [('foo', '=', 1)]),
+                (['AND', ['OR', ('bar', '=', 2), ('baz', '=', 3)],
+                    ('foo', '=', 1)],
+                    [('foo', '=', 1),
+                        ['OR', ('bar', '=', 2), ('baz', '=', 3)]]),
+                (['OR', ['AND', ('bar', '=', 2), ('baz', '=', 3)],
+                    ('foo', '=', 1)],
+                    ['OR', ('foo', '=', 1),
+                        [('bar', '=', 2), ('baz', '=', 3)]]),
+                (['AND', ['AND', ('bar', '=', 2), ('baz', '=', 3)],
+                    ('foo', '=', 1)],
+                    [('bar', '=', 2), ('baz', '=', 3), ('foo', '=', 1)]),
+                (['OR', ['OR', ('bar', '=', 2), ('baz', '=', 3)],
+                    ('foo', '=', 1)],
+                    ['OR', ('bar', '=', 2), ('baz', '=', 3), ('foo', '=', 1)]),
+                ]:
+            with self.subTest(domain=domain):
+                self.assertEqual(canonicalize(domain), expected)
 
 
-def suite():
-    func = unittest.TestLoader().loadTestsFromTestCase
-    suite = unittest.TestSuite()
-    for testcase in [ToolsTestCase,
-            StringPartitionedTestCase,
-            DomainInversionTestCase]:
-        suite.addTests(func(testcase))
-    suite.addTest(doctest.DocTestSuite(decimal_))
-    return suite
+@unittest.skipUnless(barcode, "required barcode")
+class BarcodeTestCase(unittest.TestCase):
+    "Test barcode module"
+
+    def test_generate_svg(self):
+        "Test generate SVG"
+        image = barcode.generate_svg('ean', "8749903790831")
+        self.assertIsInstance(image, BytesIO)
+        self.assertIsNotNone(image.getvalue())
+
+    def test_generate_png(self):
+        "Test generate PNG"
+        image = barcode.generate_png('ean', "8749903790831")
+        self.assertIsInstance(image, BytesIO)
+        self.assertIsNotNone(image.getvalue())
+
+
+@unittest.skipUnless(qrcode, "required qrcode")
+class QRCodeTestCase(unittest.TestCase):
+    "Test qrcode module"
+
+    def test_generate_svg(self):
+        "Test generate SVG"
+        image = qrcode.generate_svg("Tryton")
+        self.assertIsInstance(image, BytesIO)
+        self.assertIsNotNone(image.getvalue())
+
+    def test_generate_png(self):
+        "Test generate PNG"
+        image = qrcode.generate_png("Tryton")
+        self.assertIsInstance(image, BytesIO)
+        self.assertIsNotNone(image.getvalue())
+
+
+def load_tests(loader, tests, pattern):
+    tests.addTest(doctest.DocTestSuite(decimal_))
+    return tests
