@@ -5,20 +5,20 @@ __all__ = ['Wizard',
     'StateView', 'StateTransition', 'StateAction', 'StateReport',
     'Button']
 
-import json
 import copy
+import json
 
 from trytond.i18n import gettext
-from trytond.pool import Pool, PoolBase
-from trytond.transaction import Transaction
-from trytond.url import URLMixin
-from trytond.protocols.jsonrpc import JSONDecoder, JSONEncoder
 from trytond.model import ModelSQL
 from trytond.model.exceptions import AccessError
 from trytond.model.fields import states_validate
+from trytond.pool import Pool, PoolBase
+from trytond.protocols.jsonrpc import JSONDecoder, JSONEncoder
 from trytond.pyson import PYSONEncoder
 from trytond.rpc import RPC
 from trytond.tools import cached_property
+from trytond.transaction import Transaction, check_access
+from trytond.url import URLMixin
 
 
 class Button(object):
@@ -93,18 +93,32 @@ class StateView(State):
         default = getattr(wizard, 'default_%s' % state_name, None)
         if default:
             defaults.update(default(fields))
-            for field_name, value in list(defaults.items()):
-                if '.' in field_name:
-                    continue
-                field = Model_._fields[field_name]
-                if value and field._type == 'many2one':
-                    Target = pool.get(field.model_name)
-                    if 'rec_name' in Target._fields:
-                        defaults.setdefault(
-                            field_name + '.', {})['rec_name'] = Target(
-                                value).rec_name
+            self._complete_values(defaults)
         return defaults
 
+    def get_values(self, wizard, state_name, fields):
+        "Return values for the fields"
+        values = {}
+        value = getattr(wizard, 'value_%s' % state_name, None)
+        if value:
+            values.update(value(fields))
+        self._complete_values(values)
+        return values
+
+    def _complete_values(self, values):
+        pool = Pool()
+        Model_ = pool.get(self.model_name)
+        for field_name, value in list(values.items()):
+            if '.' in field_name:
+                continue
+            field = Model_._fields[field_name]
+            if value and field._type == 'many2one':
+                Target = field.get_target()
+                if 'rec_name' in Target._fields:
+                    values.setdefault(
+                        field_name + '.', {})['rec_name'] = Target(
+                            value).rec_name
+
     def get_buttons(self, wizard, state_name):
         '''
         Returns button definitions translated
@@ -200,8 +214,18 @@ class Wizard(URLMixin, PoolBase):
 
         # Copy states
         for attr in dir(cls):
-            if isinstance(getattr(cls, attr), State):
-                setattr(cls, attr, copy.deepcopy(getattr(cls, attr)))
+            if not isinstance(getattr(cls, attr), State):
+                continue
+            state_name = attr
+            state = getattr(cls, state_name)
+            # Copy the original state definition to prevent side-effect with
+            # the mutable attributes
+            for parent_cls in cls.__mro__:
+                parent_state = getattr(parent_cls, state_name, None)
+                if isinstance(parent_state, State):
+                    state = parent_state
+            state = copy.deepcopy(state)
+            setattr(cls, attr, state)
 
     @classmethod
     def __post_setup__(cls):
@@ -232,7 +256,7 @@ class Wizard(URLMixin, PoolBase):
         if Transaction().user == 0:
             return
 
-        with Transaction().set_context(_check_access=True):
+        with check_access():
             model = context.get('active_model')
             if model:
                 Model = pool.get(model)
@@ -300,8 +324,11 @@ class Wizard(URLMixin, PoolBase):
             - ``view``: a dictionary with:
                 - ``fields_view``: a fields/view definition
                 - ``defaults``: a dictionary with default values
+                - ``values``: a dictionary with values
                 - ``buttons``: a list of buttons
         '''
+        transaction = Transaction()
+        counter = transaction.counter
         cls.check_access()
         wizard = cls(session_id)
         for key, values in data.items():
@@ -310,7 +337,14 @@ class Wizard(URLMixin, PoolBase):
                 if field == 'id':
                     continue
                 setattr(record, field, value)
-        return wizard._execute(state_name)
+        result = wizard._execute(state_name)
+        # Log before _save increases the counter
+        if transaction.counter != counter and wizard.model:
+            wizard.model.log(
+                wizard.records, 'wizard',
+                f'{cls.__name__}:{state_name}')
+        wizard._save()
+        return result
 
     def _execute(self, state_name):
         if state_name == self.end_state:
@@ -321,12 +355,14 @@ class Wizard(URLMixin, PoolBase):
 
         if isinstance(state, StateView):
             view = state.get_view(self, state_name)
-            defaults = state.get_defaults(self, state_name,
-                list(view['fields'].keys()))
+            fields = list(view['fields'].keys())
+            defaults = state.get_defaults(self, state_name, fields)
+            values = state.get_values(self, state_name, fields)
             buttons = state.get_buttons(self, state_name)
             result['view'] = {
                 'fields_view': view,
                 'defaults': defaults,
+                'values': values,
                 'buttons': buttons,
                 'state': state_name,
                 }
@@ -344,7 +380,6 @@ class Wizard(URLMixin, PoolBase):
                 result = self._execute(transition())
             if do_result:
                 result.setdefault('actions', []).append(do_result)
-        self._save()
         return result
 
     def __init__(self, session_id):
